@@ -27,7 +27,7 @@ pub fn command(paths: &ProjectPaths, packages_to_add: Vec<String>, dev: bool) ->
 
     // Insert the new packages into the manifest and perform dependency
     // resolution to determine suitable versions
-    let manifest = dependencies::download(
+    let manifest = dependencies::resolve_and_download(
         paths,
         cli::Reporter::new(),
         Some((new_package_requirements.clone(), dev)),
@@ -70,16 +70,18 @@ pub fn command(paths: &ProjectPaths, packages_to_add: Vec<String>, dev: bool) ->
         #[allow(clippy::indexing_slicing)]
         {
             if dev {
+                if !gleam_toml.as_table().contains_key("dev-dependencies") {
+                    gleam_toml["dev-dependencies"] = toml_edit::table();
+                }
                 gleam_toml["dev-dependencies"][&added_package] = toml_edit::value(range.clone());
             } else {
+                if !gleam_toml.as_table().contains_key("dependencies") {
+                    gleam_toml["dependencies"] = toml_edit::table();
+                }
                 gleam_toml["dependencies"][&added_package] = toml_edit::value(range.clone());
             };
-            manifest_toml["requirements"][&added_package]
-                .as_inline_table_mut()
-                .expect("Invalid manifest format")["version"] = range.into();
+            manifest_toml["requirements"][&added_package]["version"] = range.into();
         }
-
-        cli::print_added(&format!("{added_package} v{version}"));
     }
 
     // Write the updated config
