@@ -1,8 +1,10 @@
 use gleam_core::{
     build::Telemetry,
     error::{Error, StandardIoAction},
+    manifest::{Changed, ChangedGit, Resolved},
 };
 use hexpm::version::Version;
+use itertools::Itertools as _;
 use std::{
     io::{IsTerminal, Write},
     time::{Duration, Instant},
@@ -54,6 +56,10 @@ impl Telemetry for Reporter {
     fn waiting_for_build_directory_lock(&self) {
         print_waiting_for_build_directory_lock()
     }
+
+    fn resolved_package_versions(&self, resolved: &Resolved) {
+        print_resolved(resolved)
+    }
 }
 
 pub fn ask(question: &str) -> Result<String, Error> {
@@ -148,11 +154,35 @@ pub(crate) fn print_running(text: &str) {
     print_colourful_prefix("Running", text)
 }
 
-pub(crate) fn print_added(text: &str) {
+pub(crate) fn print_resolved(resolved: &Resolved) {
+    for (name, version) in resolved.added.iter().sorted() {
+        print_added(&format!("{name} v{version}"));
+    }
+    for Changed { name, old, new } in resolved.changed.iter().sorted_by_key(|p| &p.name) {
+        print_changed(&format!("{name} v{old} -> v{new}"));
+    }
+    for ChangedGit {
+        name,
+        old_hash,
+        new_hash,
+    } in resolved.changed_git.iter().sorted_by_key(|p| &p.name)
+    {
+        print_changed(&format!("{name} {old_hash} -> {new_hash}"));
+    }
+    for name in resolved.removed.iter().sorted() {
+        print_removed(name);
+    }
+}
+
+fn print_added(text: &str) {
     print_colourful_prefix("Added", text)
 }
 
-pub(crate) fn print_removed(text: &str) {
+fn print_changed(text: &str) {
+    print_colourful_prefix("Changed", text)
+}
+
+fn print_removed(text: &str) {
     print_colourful_prefix("Removed", text)
 }
 
@@ -160,6 +190,14 @@ pub(crate) fn print_generating_documentation() {
     print_colourful_prefix("Generating", "documentation")
 }
 
+pub(crate) fn print_transferring_ownership() {
+    print_colourful_prefix("Transferring", "ownership");
+}
+
+pub(crate) fn print_transferred_ownership() {
+    print_colourful_prefix("Transferred", "ownership");
+}
+
 fn print_packages_downloaded(start: Instant, count: usize) {
     let elapsed = seconds(start.elapsed());
     let msg = match count {
