@@ -19,7 +19,7 @@ use gleam_core::{
     error::{FileIoAction, FileKind, ShellCommandFailureReason, StandardIoAction},
     hex::{self, HEXPM_PUBLIC_KEY},
     io::{HttpClient as _, TarUnpacker, WrappedReader},
-    manifest::{Base16Checksum, Manifest, ManifestPackage, ManifestPackageSource},
+    manifest::{Base16Checksum, Manifest, ManifestPackage, ManifestPackageSource, Resolved},
     paths::ProjectPaths,
     requirement::Requirement,
 };
@@ -104,7 +104,9 @@ fn get_manifest_details(paths: &ProjectPaths) -> Result<(PackageConfig, Manifest
         cli::Reporter::new(),
         Mode::Dev,
     );
-    let (_, manifest) = dependency_manager.get_manifest(paths, &config, Vec::new())?;
+    let manifest = dependency_manager
+        .resolve_versions(paths, &config, Vec::new())?
+        .manifest;
     Ok((config, manifest))
 }
 
@@ -226,7 +228,7 @@ pub fn update(paths: &ProjectPaths, packages: Vec<String>) -> Result<()> {
     };
 
     // Update specific packages
-    _ = download(
+    _ = resolve_and_download(
         paths,
         cli::Reporter::new(),
         None,
@@ -255,7 +257,8 @@ pub fn cleanup<Telem: Telemetry>(paths: &ProjectPaths, telemetry: Telem) -> Resu
 
     // Read the project config
     let config = crate::root_config(paths)?;
-    let mut manifest = read_manifest_from_disc(paths)?;
+    let old_manifest = read_manifest_from_disc(paths)?;
+    let mut manifest = old_manifest.clone();
 
     remove_extra_requirements(&config, &mut manifest)?;
 
@@ -268,7 +271,10 @@ pub fn cleanup<Telem: Telemetry>(paths: &ProjectPaths, telemetry: Telem) -> Resu
     write_manifest_to_disc(paths, &manifest)?;
     LocalPackages::from_manifest(&manifest).write_to_disc(paths)?;
 
-    Ok(manifest)
+    let resolved = Resolved::with_updates(&old_manifest, manifest);
+    telemetry.resolved_package_versions(&resolved);
+
+    Ok(resolved.manifest)
 }
 
 /// Remove requirements and unneeded packages from manifest that are no longer present in config.
@@ -365,7 +371,7 @@ pub fn parse_gleam_add_specifier(package: &str) -> Result<(EcoString, Requiremen
     Ok((package.into(), requirement))
 }
 
-pub fn download<Telem: Telemetry>(
+pub fn resolve_and_download<Telem: Telemetry>(
     paths: &ProjectPaths,
     telemetry: Telem,
     new_package: Option<(Vec<(EcoString, Requirement)>, bool)>,
@@ -383,7 +389,7 @@ pub fn download<Telem: Telemetry>(
         Mode::Dev,
     );
 
-    dependency_manager.download(paths, new_package, packages_to_update)
+    dependency_manager.resolve_and_download_versions(paths, new_package, packages_to_update)
 }
 
 fn pretty_print_major_versions_available(versions: dependency::PackageVersionDiffs) -> String {
@@ -848,7 +854,7 @@ fn download_git_package(
     // remove the directory because running `git init` in a non-empty directory
     // followed by `git checkout ...` is an error. See
     // https://github.com/gleam-lang/gleam/issues/4488 for details.
-    if !fs::is_inside_git_work_tree(&package_path).unwrap_or(false) {
+    if !fs::is_git_work_tree_root(&package_path) {
         fs::delete_directory(&package_path)?;
     }
 
@@ -1037,11 +1043,11 @@ pub fn unlock_packages(
         let mut packages_to_unlock: Vec<EcoString> = packages_to_unlock.to_vec();
 
         while let Some(package_name) = packages_to_unlock.pop() {
-            if locked.remove(&package_name).is_some() {
-                if let Some(package) = manifest.packages.iter().find(|p| p.name == package_name) {
-                    let deps_to_unlock = find_deps_to_unlock(package, locked, manifest);
-                    packages_to_unlock.extend(deps_to_unlock);
-                }
+            if locked.remove(&package_name).is_some()
+                && let Some(package) = manifest.packages.iter().find(|p| p.name == package_name)
+            {
+                let deps_to_unlock = find_deps_to_unlock(package, locked, manifest);
+                packages_to_unlock.extend(deps_to_unlock);
             }
         }
     } else {
@@ -1177,13 +1183,13 @@ impl dependency::PackageFetcher for PackageFetcher {
 
         tracing::debug!(package = package, "looking_up_hex_package");
         let config = hexpm::Config::new();
-        let request = hexpm::get_package_request(package, None, &config);
+        let request = hexpm::repository_v2_get_package_request(package, None, &config);
         let response = self
             .runtime
             .block_on(self.http.send(request))
             .map_err(PackageFetchError::fetch_error)?;
 
-        let pkg = hexpm::get_package_response(response, HEXPM_PUBLIC_KEY)
+        let pkg = hexpm::repository_v2_get_package_response(response, HEXPM_PUBLIC_KEY)
             .map_err(PackageFetchError::from)?;
         let pkg = Rc::new(pkg);
         let pkg_ref = Rc::clone(&pkg);
