@@ -4,6 +4,7 @@ use flate2::{Compression, write::GzEncoder};
 use gleam_core::{
     Error, Result,
     analyse::TargetSupport,
+    ast::{CallArg, Statement, TypedExpr, TypedFunction},
     build::{Codegen, Compile, Mode, Options, Package, Target},
     config::{GleamVersion, PackageConfig, SpdxLicense},
     docs::{Dependency, DependencyKind, DocContext},
@@ -21,6 +22,8 @@ use std::{collections::HashMap, io::Write, path::PathBuf, time::Instant};
 
 use crate::{build, cli, docs, fs, http::HttpClient};
 
+const CORE_TEAM_PUBLISH_PASSWORD: &str = "Trans rights are human rights";
+
 pub fn command(paths: &ProjectPaths, replace: bool, i_am_sure: bool) -> Result<()> {
     let mut config = crate::config::root_config(paths)?;
 
@@ -44,6 +47,7 @@ pub fn command(paths: &ProjectPaths, replace: bool, i_am_sure: bool) -> Result<(
 
     check_for_name_squatting(&compile_result)?;
     check_for_multiple_top_level_modules(&compile_result, i_am_sure)?;
+    check_for_default_main(&compile_result)?;
 
     // Build HTML documentation
     let docs_tarball = fs::create_tar_archive(docs::build_documentation(
@@ -148,13 +152,70 @@ fn check_for_name_squatting(package: &Package) -> Result<(), Error> {
         return Ok(());
     };
 
-    if main.body.first().is_println() {
+    if let Some(first) = &main.body.first()
+        && first.is_println()
+    {
         return Err(Error::HexPackageSquatting);
     }
 
     Ok(())
 }
 
+/// Checks if publishing packages contain default main functions.
+/// Main functions with documentation are considered intentional and allowed.
+fn check_for_default_main(package: &Package) -> Result<(), Error> {
+    let package_name = &package.config.name;
+
+    let has_default_main = package
+        .modules
+        .iter()
+        .flat_map(|m| m.ast.definitions.iter())
+        .filter_map(|d| d.main_function())
+        .any(|main| main.documentation.is_none() && is_default_main(main, package_name));
+
+    if has_default_main {
+        return Err(Error::CannotPublishWithDefaultMain {
+            package_name: package_name.clone(),
+        });
+    }
+
+    Ok(())
+}
+
+fn is_default_main(main: &TypedFunction, package_name: &EcoString) -> bool {
+    if main.body.len() != 1 {
+        return false;
+    }
+
+    let Some(Statement::Expression(expression)) = main.body.first() else {
+        return false;
+    };
+
+    if !expression.is_println() {
+        return false;
+    }
+
+    match expression {
+        TypedExpr::Call { arguments, .. } => {
+            if arguments.len() != 1 {
+                return false;
+            }
+
+            match arguments.first() {
+                Some(CallArg {
+                    value: TypedExpr::String { value, .. },
+                    ..
+                }) => {
+                    let default_argument = format!("Hello from {}!", &package_name);
+                    value == &default_argument
+                }
+                _ => false,
+            }
+        }
+        _ => false,
+    }
+}
+
 fn check_for_multiple_top_level_modules(package: &Package, i_am_sure: bool) -> Result<(), Error> {
     // Collect top-level module names
     let mut top_level_module_names = package
@@ -264,9 +325,9 @@ fn check_for_gleam_prefix(config: &PackageConfig) -> Result<bool, Error> {
 the prefix `gleam_`, which is for packages maintained by the Gleam
 core team.\n",
     );
-    let should_publish = cli::confirm_with_text("I am part of the Gleam core team")?;
+    let password = cli::ask_password("Please enter the core team password to continue")?;
     println!();
-    Ok(should_publish)
+    Ok(password == CORE_TEAM_PUBLISH_PASSWORD)
 }
 
 struct Tarball {
@@ -342,17 +403,13 @@ fn do_build_hex_tarball(paths: &ProjectPaths, config: &mut PackageConfig) -> Res
         // Otherwise we need to check that the annotated version range is
         // correct and includes the minimum required version.
         Some(gleam_version) => {
-            if let Some(lowest_allowed_version) = gleam_version.lowest_version() {
-                if lowest_allowed_version < minimum_required_version {
-                    return Err(Error::CannotPublishWrongVersion {
-                        minimum_required_version: SmallVersion::from_hexpm(
-                            minimum_required_version,
-                        ),
-                        wrongfully_allowed_version: SmallVersion::from_hexpm(
-                            lowest_allowed_version,
-                        ),
-                    });
-                }
+            if let Some(lowest_allowed_version) = gleam_version.lowest_version()
+                && lowest_allowed_version < minimum_required_version
+            {
+                return Err(Error::CannotPublishWrongVersion {
+                    minimum_required_version: SmallVersion::from_hexpm(minimum_required_version),
+                    wrongfully_allowed_version: SmallVersion::from_hexpm(lowest_allowed_version),
+                });
             }
         }
     }
