@@ -18,6 +18,7 @@ use crate::{
     call_graph::{CallGraphNode, into_dependency_order},
     config::PackageConfig,
     dep_tree,
+    inline::{self, InlinableFunction},
     line_numbers::LineNumbers,
     parse::SpannedString,
     reference::{EntityKind, ReferenceKind},
@@ -37,12 +38,12 @@ use crate::{
     warning::TypeWarningEmitter,
 };
 use camino::Utf8PathBuf;
-use ecow::EcoString;
+use ecow::{EcoString, eco_format};
 use hexpm::version::Version;
 use itertools::Itertools;
 use name::{check_argument_names, check_name_case};
 use std::{
-    collections::HashMap,
+    collections::{HashMap, HashSet},
     ops::Deref,
     sync::{Arc, OnceLock},
 };
@@ -145,6 +146,7 @@ pub struct ModuleAnalyzerConstructor<'a, A> {
     pub importable_modules: &'a im::HashMap<EcoString, ModuleInterface>,
     pub warnings: &'a TypeWarningEmitter,
     pub direct_dependencies: &'a HashMap<EcoString, A>,
+    pub dev_dependencies: &'a HashSet<EcoString>,
     pub target_support: TargetSupport,
     pub package_config: &'a PackageConfig,
 }
@@ -166,6 +168,7 @@ impl<A> ModuleAnalyzerConstructor<'_, A> {
             importable_modules: self.importable_modules,
             warnings: self.warnings,
             direct_dependencies: self.direct_dependencies,
+            dev_dependencies: self.dev_dependencies,
             target_support: self.target_support,
             package_config: self.package_config,
             line_numbers,
@@ -174,6 +177,7 @@ impl<A> ModuleAnalyzerConstructor<'_, A> {
             value_names: HashMap::with_capacity(module.definitions.len()),
             hydrators: HashMap::with_capacity(module.definitions.len()),
             module_name: module.name.clone(),
+            inline_functions: HashMap::new(),
             minimum_required_version: Version::new(0, 1, 0),
         }
         .infer_module(module)
@@ -187,6 +191,7 @@ struct ModuleAnalyzer<'a, A> {
     importable_modules: &'a im::HashMap<EcoString, ModuleInterface>,
     warnings: &'a TypeWarningEmitter,
     direct_dependencies: &'a HashMap<EcoString, A>,
+    dev_dependencies: &'a HashSet<EcoString>,
     target_support: TargetSupport,
     package_config: &'a PackageConfig,
     line_numbers: LineNumbers,
@@ -196,6 +201,8 @@ struct ModuleAnalyzer<'a, A> {
     hydrators: HashMap<EcoString, Hydrator>,
     module_name: EcoString,
 
+    inline_functions: HashMap<EcoString, InlinableFunction>,
+
     /// The minimum Gleam version required to compile the analysed module.
     minimum_required_version: Version,
 }
@@ -220,6 +227,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             importable_modules: self.importable_modules,
             target_support: self.target_support,
             current_origin: self.origin,
+            dev_dependencies: self.dev_dependencies,
         }
         .build();
 
@@ -372,6 +380,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                     value_references: env.references.value_references,
                     type_references: env.references.type_references,
                 },
+                inline_functions: self.inline_functions,
             },
         };
 
@@ -510,7 +519,10 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         } = f;
         let (name_location, name) = name.expect("Function in a definition must be named");
         let target = environment.target;
-        let body_location = body.last().location();
+        let body_location = body
+            .last()
+            .map(|statement| statement.location())
+            .unwrap_or(location);
         let preregistered_fn = environment
             .get_variable(&name)
             .expect("Could not find preregistered type for function");
@@ -544,38 +556,42 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             self.ensure_annotations_present(&arguments, return_annotation.as_ref(), location);
         }
 
-        let has_body = !body.first().is_placeholder();
+        let has_body = !body.is_empty();
         let definition = FunctionDefinition {
             has_body,
             has_erlang_external: external_erlang.is_some(),
             has_javascript_external: external_javascript.is_some(),
         };
 
-        let mut typed_arguments = Vec::with_capacity(arguments.len());
-        for (argument, type_) in arguments.into_iter().zip(&prereg_arguments_types) {
-            let argument = argument.set_type(type_.clone());
-            match &argument.names {
-                ast::ArgNames::Named { .. } | ast::ArgNames::NamedLabelled { .. } => (),
-                ast::ArgNames::Discard { name, location }
-                | ast::ArgNames::LabelledDiscard {
-                    name,
-                    name_location: location,
-                    ..
-                } => {
-                    let _ = environment.discarded_names.insert(name.clone(), *location);
-                }
-            }
-
-            typed_arguments.push(argument);
-        }
-
         // We have already registered the function in the `register_value_from_function`
         // method, but here we must set this as the current function again, so that anything
         // we reference in the body of it can be tracked properly in the call graph.
         environment.references.set_current_node(name.clone());
 
+        let mut typed_arguments = Vec::with_capacity(arguments.len());
+
         // Infer the type using the preregistered args + return types as a starting point
         let result = environment.in_new_scope(&mut self.problems, |environment, problems| {
+            for (argument, type_) in arguments.into_iter().zip(&prereg_arguments_types) {
+                let argument = argument.set_type(type_.clone());
+
+                // We track which arguments are discarded so we can provide nice
+                // error messages when someone
+                match &argument.names {
+                    ast::ArgNames::Named { .. } | ast::ArgNames::NamedLabelled { .. } => (),
+                    ast::ArgNames::Discard { name, location }
+                    | ast::ArgNames::LabelledDiscard {
+                        name,
+                        name_location: location,
+                        ..
+                    } => {
+                        let _ = environment.discarded_names.insert(name.clone(), *location);
+                    }
+                }
+
+                typed_arguments.push(argument);
+            }
+
             let mut expr_typer = ExprTyper::new(environment, definition, problems);
             expr_typer.hydrator = self
                 .hydrators
@@ -583,12 +599,17 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 .expect("Could not find hydrator for fn");
 
             let (arguments, body) = expr_typer.infer_fn_with_known_types(
+                Some(name.clone()),
                 typed_arguments.clone(),
                 body,
                 Some(prereg_return_type.clone()),
             )?;
             let arguments_types = arguments.iter().map(|a| a.type_.clone()).collect();
-            let type_ = fn_(arguments_types, body.last().type_());
+            let type_ = fn_(
+                arguments_types,
+                body.last()
+                    .map_or(prereg_return_type.clone(), |last| last.type_()),
+            );
             Ok((
                 type_,
                 body,
@@ -609,13 +630,14 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             Err(error) => {
                 self.problems.error(error);
                 let type_ = preregistered_type.clone();
-                let body = Vec1::new(Statement::Expression(TypedExpr::Invalid {
+                let body = vec![Statement::Expression(TypedExpr::Invalid {
                     type_: prereg_return_type.clone(),
                     location: SrcSpan {
                         start: body_location.end,
                         end: body_location.end,
                     },
-                }));
+                    extra_information: None,
+                })];
                 let implementations = Implementations::supporting_all();
                 (
                     type_,
@@ -643,10 +665,10 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             } => self.track_feature_usage(FeatureKind::InternalAnnotation, location),
         }
 
-        if let Some((module, _, location)) = &external_javascript {
-            if module.contains('@') {
-                self.track_feature_usage(FeatureKind::AtInJavascriptModules, *location)
-            }
+        if let Some((module, _, location)) = &external_javascript
+            && module.contains('@')
+        {
+            self.track_feature_usage(FeatureKind::AtInJavascriptModules, *location)
         }
 
         // Assert that the inferred type matches the type of any recursive call
@@ -708,10 +730,10 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             ReferenceKind::Definition,
         );
 
-        Definition::Function(Function {
+        let function = Function {
             documentation: doc,
             location,
-            name: Some((name_location, name)),
+            name: Some((name_location, name.clone())),
             publicity,
             deprecation,
             arguments: typed_arguments,
@@ -726,7 +748,17 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             external_javascript,
             implementations,
             purity,
-        })
+        };
+
+        if let Some(inline_function) = inline::function_to_inlinable(
+            &environment.current_package,
+            &environment.current_module,
+            &function,
+        ) {
+            _ = self.inline_functions.insert(name, inline_function);
+        }
+
+        Definition::Function(function)
     }
 
     fn assert_valid_javascript_external(
@@ -791,13 +823,13 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
 
     fn ensure_function_has_an_implementation(
         &mut self,
-        body: &Vec1<UntypedStatement>,
+        body: &[UntypedStatement],
         external_erlang: &Option<(EcoString, EcoString, SrcSpan)>,
         external_javascript: &Option<(EcoString, EcoString, SrcSpan)>,
         location: SrcSpan,
     ) -> bool {
         match (external_erlang, external_javascript) {
-            (None, None) if body.first().is_placeholder() => {
+            (None, None) if body.is_empty() => {
                 self.problems.error(Error::NoImplementation { location });
                 false
             }
@@ -1080,6 +1112,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 label,
                 ast,
                 location,
+                doc,
                 ..
             } in constructor.arguments.iter()
             {
@@ -1095,6 +1128,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 fields.push(TypeValueConstructorField {
                     type_: t.clone(),
                     label: label.as_ref().map(|(_location, label)| label.clone()),
+                    documentation: doc.as_ref().map(|(_, documentation)| documentation.clone()),
                 });
 
                 // Register the type for this parameter
@@ -1311,6 +1345,15 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             });
         }
 
+        if *opaque && publicity.is_private() {
+            self.problems.error(Error::PrivateOpaqueType {
+                location: SrcSpan {
+                    start: location.start,
+                    end: location.start + 6,
+                },
+            });
+        }
+
         Ok(())
     }
 
@@ -1360,7 +1403,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 TypeConstructor {
                     origin: *location,
                     module: self.module_name.clone(),
-                    parameters,
+                    parameters: parameters.clone(),
                     type_: type_.clone(),
                     deprecation: deprecation.clone(),
                     publicity: *publicity,
@@ -1368,18 +1411,24 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 },
             )?;
 
-            environment.insert_type_alias(
-                name.clone(),
-                TypeAliasConstructor {
-                    origin: *location,
-                    module: self.module_name.clone(),
-                    type_,
-                    publicity: *publicity,
-                    deprecation: deprecation.clone(),
-                    documentation: documentation.as_ref().map(|(_, doc)| doc.clone()),
-                    arity,
-                },
-            )?;
+            let alias = TypeAliasConstructor {
+                origin: *location,
+                module: self.module_name.clone(),
+                type_,
+                publicity: *publicity,
+                deprecation: deprecation.clone(),
+                documentation: documentation.as_ref().map(|(_, doc)| doc.clone()),
+                arity,
+                parameters,
+            };
+
+            environment.names.maybe_register_reexport_alias(
+                &environment.current_package,
+                name,
+                &alias,
+            );
+
+            environment.insert_type_alias(name.clone(), alias)?;
 
             if let Some(name) = hydrator.unused_type_variables().next() {
                 return Err(Error::UnusedTypeAliasParameter {
@@ -1558,21 +1607,21 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
 
         // Then if the required version is not in the specified version for the
         // range we emit a warning highlighting the usage of the feature.
-        if let Some(gleam_version) = &self.package_config.gleam_version {
-            if let Some(lowest_allowed_version) = gleam_version.lowest_version() {
-                // There is a version in the specified range that is lower than
-                // the one required by this feature! This means that the
-                // specified range is wrong and would allow someone to run a
-                // compiler that is too old to know of this feature.
-                if minimum_required_version > lowest_allowed_version {
-                    self.problems
-                        .warning(Warning::FeatureRequiresHigherGleamVersion {
-                            location,
-                            feature_kind,
-                            minimum_required_version: minimum_required_version.clone(),
-                            wrongfully_allowed_version: lowest_allowed_version,
-                        })
-                }
+        if let Some(gleam_version) = &self.package_config.gleam_version
+            && let Some(lowest_allowed_version) = gleam_version.lowest_version()
+        {
+            // There is a version in the specified range that is lower than
+            // the one required by this feature! This means that the
+            // specified range is wrong and would allow someone to run a
+            // compiler that is too old to know of this feature.
+            if minimum_required_version > lowest_allowed_version {
+                self.problems
+                    .warning(Warning::FeatureRequiresHigherGleamVersion {
+                        location,
+                        feature_kind,
+                        minimum_required_version: minimum_required_version.clone(),
+                        wrongfully_allowed_version: lowest_allowed_version,
+                    })
             }
         }
 
@@ -1894,19 +1943,12 @@ struct Accessors {
 }
 
 fn custom_type_accessors(constructors: &[TypeValueConstructor]) -> Result<Accessors, Error> {
-    let arguments = get_compatible_record_fields(constructors);
+    let accessors = get_compatible_record_fields(constructors);
 
-    let mut shared_accessors = HashMap::with_capacity(arguments.len());
+    let mut shared_accessors = HashMap::with_capacity(accessors.len());
 
-    for (index, label, type_) in arguments {
-        let _ = shared_accessors.insert(
-            label.clone(),
-            RecordAccessor {
-                index: index as u64,
-                label: label.clone(),
-                type_: type_.clone(),
-            },
-        );
+    for accessor in accessors {
+        let _ = shared_accessors.insert(accessor.label.clone(), accessor);
     }
 
     let mut variant_specific_accessors: Vec<HashMap<EcoString, RecordAccessor>> =
@@ -1926,6 +1968,7 @@ fn custom_type_accessors(constructors: &[TypeValueConstructor]) -> Result<Access
                     index: index as u64,
                     label: label.clone(),
                     type_: parameter.type_.clone(),
+                    documentation: parameter.documentation.clone(),
                 },
             );
         }
@@ -1940,9 +1983,7 @@ fn custom_type_accessors(constructors: &[TypeValueConstructor]) -> Result<Access
 
 /// Returns the fields that have the same label and type across all variants of
 /// the given type.
-fn get_compatible_record_fields(
-    constructors: &[TypeValueConstructor],
-) -> Vec<(usize, &EcoString, &Arc<Type>)> {
+fn get_compatible_record_fields(constructors: &[TypeValueConstructor]) -> Vec<RecordAccessor> {
     let mut compatible = vec![];
 
     let first = match constructors.first() {
@@ -1957,6 +1998,21 @@ fn get_compatible_record_fields(
             None => continue 'next_argument,
         };
 
+        let mut documentation = if constructors.len() == 1 {
+            // If there is only one constructor, we simply show the documentation
+            // for the field.
+            first_parameter.documentation.clone()
+        } else {
+            // If there are multiple constructors, we show the documentation of
+            // this field for each of the variants.
+            first_parameter
+                .documentation
+                .as_ref()
+                .map(|field_documentation| {
+                    eco_format!("## {}\n\n{}", first.name, field_documentation)
+                })
+        };
+
         // Check each variant to see if they have an field in the same position
         // with the same label and the same type
         for constructor in constructors.iter().skip(1) {
@@ -1979,12 +2035,33 @@ fn get_compatible_record_fields(
             if !parameter.type_.same_as(&first_parameter.type_) {
                 continue 'next_argument;
             }
+
+            if let Some(field_documentation) = &parameter.documentation {
+                let field_documentation =
+                    eco_format!("## {}\n\n{}", constructor.name, field_documentation);
+
+                match &mut documentation {
+                    None => {
+                        documentation = Some(field_documentation);
+                    }
+                    Some(documentation) => {
+                        documentation.push('\n');
+                        documentation.push_str(&field_documentation);
+                    }
+                }
+            }
         }
 
         // The previous loop did not find any incompatible fields in the other
         // variants so this field is compatible across variants and we should
         // generate an accessor for it.
-        compatible.push((index, first_label, &first_parameter.type_))
+
+        compatible.push(RecordAccessor {
+            index: index as u64,
+            label: first_label.clone(),
+            type_: first_parameter.type_.clone(),
+            documentation,
+        })
     }
 
     compatible
