@@ -38,7 +38,7 @@ pub enum Constant<T, RecordTag> {
         location: SrcSpan,
         module: Option<(EcoString, SrcSpan)>,
         name: EcoString,
-        args: Vec<CallArg<Self>>,
+        arguments: Vec<CallArg<Self>>,
         tag: RecordTag,
         type_: T,
         field_map: Option<FieldMap>,
@@ -103,7 +103,7 @@ impl TypedConstant {
                 .iter()
                 .find_map(|element| element.find_node(byte_index))
                 .unwrap_or(Located::Constant(self)),
-            Constant::Record { args, .. } => args
+            Constant::Record { arguments, .. } => arguments
                 .iter()
                 .find_map(|argument| argument.find_node(byte_index))
                 .unwrap_or(Located::Constant(self)),
@@ -141,24 +141,24 @@ impl TypedConstant {
         }
     }
 
-    pub(crate) fn referenced_variables(&self) -> HashSet<&EcoString> {
+    pub(crate) fn referenced_variables(&self) -> im::HashSet<&EcoString> {
         match self {
-            Constant::Var { name, .. } => hashset![name],
+            Constant::Var { name, .. } => im::hashset![name],
 
             Constant::Invalid { .. }
             | Constant::Int { .. }
             | Constant::Float { .. }
-            | Constant::String { .. } => hashset![],
+            | Constant::String { .. } => im::hashset![],
 
             Constant::List { elements, .. } | Constant::Tuple { elements, .. } => elements
                 .iter()
                 .map(|element| element.referenced_variables())
-                .fold(hashset![], HashSet::union),
+                .fold(im::hashset![], im::HashSet::union),
 
-            Constant::Record { args, .. } => args
+            Constant::Record { arguments, .. } => arguments
                 .iter()
-                .map(|arg| arg.value.referenced_variables())
-                .fold(hashset![], HashSet::union),
+                .map(|argument| argument.value.referenced_variables())
+                .fold(im::hashset![], im::HashSet::union),
 
             Constant::BitArray { segments, .. } => segments
                 .iter()
@@ -167,9 +167,9 @@ impl TypedConstant {
                         .options
                         .iter()
                         .map(|option| option.referenced_variables())
-                        .fold(segment.value.referenced_variables(), HashSet::union)
+                        .fold(segment.value.referenced_variables(), im::HashSet::union)
                 })
-                .fold(hashset![], HashSet::union),
+                .fold(im::hashset![], im::HashSet::union),
 
             Constant::StringConcatenation { left, right, .. } => left
                 .referenced_variables()
@@ -200,11 +200,21 @@ impl<A, B> Constant<A, B> {
         }
     }
 
-    pub fn is_simple(&self) -> bool {
-        matches!(
-            self,
-            Self::Int { .. } | Self::Float { .. } | Self::String { .. }
-        )
+    #[must_use]
+    pub fn can_have_multiple_per_line(&self) -> bool {
+        match self {
+            Constant::Int { .. }
+            | Constant::Float { .. }
+            | Constant::String { .. }
+            | Constant::Var { .. } => true,
+
+            Constant::Tuple { .. }
+            | Constant::List { .. }
+            | Constant::Record { .. }
+            | Constant::BitArray { .. }
+            | Constant::StringConcatenation { .. }
+            | Constant::Invalid { .. } => false,
+        }
     }
 }
 
