@@ -102,6 +102,7 @@ pub enum TypedExpr {
         label: EcoString,
         index: u64,
         record: Box<Self>,
+        documentation: Option<EcoString>,
     },
 
     ModuleSelect {
@@ -187,6 +188,10 @@ pub enum TypedExpr {
     Invalid {
         location: SrcSpan,
         type_: Arc<Type>,
+        /// Extra information about the invalid expression, useful for providing
+        /// addition help or information, such as code actions to fix invalid
+        /// states.
+        extra_information: Option<InvalidExpression>,
     },
 }
 
@@ -339,10 +344,10 @@ impl TypedExpr {
                     }
                 }
 
-                if let Some(tail) = tail {
-                    if let Some(node) = tail.find_node(byte_index) {
-                        return Some(node);
-                    }
+                if let Some(tail) = tail
+                    && let Some(node) = tail.find_node(byte_index)
+                {
+                    return Some(node);
                 }
                 self.self_if_contains_location(byte_index)
             }
@@ -484,10 +489,10 @@ impl TypedExpr {
                     }
                 }
 
-                if let Some(tail) = tail {
-                    if let Some(node) = tail.find_statement(byte_index) {
-                        return Some(node);
-                    }
+                if let Some(tail) = tail
+                    && let Some(node) = tail.find_statement(byte_index)
+                {
+                    return Some(node);
                 }
                 None
             }
@@ -785,6 +790,7 @@ impl TypedExpr {
         match self {
             TypedExpr::Var { constructor, .. } => constructor.get_documentation(),
             TypedExpr::ModuleSelect { constructor, .. } => constructor.get_documentation(),
+            TypedExpr::RecordAccess { documentation, .. } => documentation.as_deref(),
 
             TypedExpr::Int { .. }
             | TypedExpr::Float { .. }
@@ -803,7 +809,6 @@ impl TypedExpr {
             | TypedExpr::Panic { .. }
             | TypedExpr::BitArray { .. }
             | TypedExpr::RecordUpdate { .. }
-            | TypedExpr::RecordAccess { .. }
             | TypedExpr::NegateBool { .. }
             | TypedExpr::NegateInt { .. }
             | TypedExpr::Invalid { .. } => None,
@@ -1028,6 +1033,15 @@ impl TypedExpr {
         }
     }
 
+    pub fn var_constructor(&self) -> Option<(&ValueConstructor, &EcoString)> {
+        match self {
+            TypedExpr::Var {
+                constructor, name, ..
+            } => Some((constructor, name)),
+            _ => None,
+        }
+    }
+
     #[must_use]
     pub(crate) fn is_panic(&self) -> bool {
         match self {
@@ -1161,3 +1175,11 @@ impl bit_array::GetLiteralValue for TypedExpr {
         }
     }
 }
+
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub enum InvalidExpression {
+    ModuleSelect {
+        module_name: EcoString,
+        label: EcoString,
+    },
+}
