@@ -40,7 +40,7 @@
 
 use crate::{
     analyse::Inferred,
-    ast::{BitArraySize, TypedBitArraySize},
+    ast::{BitArraySize, TypedBitArraySize, typed::InvalidExpression},
     exhaustiveness::CompiledCase,
     type_::{
         ModuleValueConstructor, PatternConstructor, TypedCallArg, ValueConstructor,
@@ -223,6 +223,7 @@ pub trait Visit<'ast> {
         visit_typed_expr_case(self, location, type_, subjects, clauses, compiled_case);
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn visit_typed_expr_record_access(
         &mut self,
         location: &'ast SrcSpan,
@@ -231,8 +232,18 @@ pub trait Visit<'ast> {
         label: &'ast EcoString,
         index: &'ast u64,
         record: &'ast TypedExpr,
+        documentation: &'ast Option<EcoString>,
     ) {
-        visit_typed_expr_record_access(self, location, field_start, type_, label, index, record);
+        visit_typed_expr_record_access(
+            self,
+            location,
+            field_start,
+            type_,
+            label,
+            index,
+            record,
+            documentation,
+        );
     }
 
     #[allow(clippy::too_many_arguments)]
@@ -324,8 +335,13 @@ pub trait Visit<'ast> {
         visit_typed_expr_negate_int(self, location, value)
     }
 
-    fn visit_typed_expr_invalid(&mut self, location: &'ast SrcSpan, type_: &'ast Arc<Type>) {
-        visit_typed_expr_invalid(self, location, type_);
+    fn visit_typed_expr_invalid(
+        &mut self,
+        location: &'ast SrcSpan,
+        type_: &'ast Arc<Type>,
+        extra_information: &'ast Option<InvalidExpression>,
+    ) {
+        visit_typed_expr_invalid(self, location, type_, extra_information);
     }
 
     fn visit_typed_statement(&mut self, stmt: &'ast TypedStatement) {
@@ -382,13 +398,13 @@ pub trait Visit<'ast> {
 
     fn visit_typed_clause_guard_field_access(
         &mut self,
-        location: &'ast SrcSpan,
+        label_location: &'ast SrcSpan,
         index: &'ast Option<u64>,
         label: &'ast EcoString,
         type_: &'ast Arc<Type>,
         container: &'ast TypedClauseGuard,
     ) {
-        visit_typed_clause_guard_field_access(self, location, index, label, type_, container)
+        visit_typed_clause_guard_field_access(self, label_location, index, label, type_, container)
     }
 
     fn visit_typed_clause_guard_module_select(
@@ -626,8 +642,17 @@ pub fn visit_typed_function<'a, V>(v: &mut V, fun: &'a TypedFunction)
 where
     V: Visit<'a> + ?Sized,
 {
-    for stmt in &fun.body {
-        v.visit_typed_statement(stmt);
+    for argument in fun.arguments.iter() {
+        if let Some(annotation) = &argument.annotation {
+            v.visit_type_ast(annotation);
+        }
+    }
+    if let Some(annotation) = &fun.return_annotation {
+        v.visit_type_ast(annotation);
+    }
+
+    for statement in &fun.body {
+        v.visit_typed_statement(statement);
     }
 }
 
@@ -642,6 +667,7 @@ where
             arguments,
             module,
             name,
+            start_parentheses: _,
         }) => {
             v.visit_type_ast_constructor(location, name_location, module, name, arguments);
         }
@@ -813,7 +839,16 @@ where
             label,
             index,
             record,
-        } => v.visit_typed_expr_record_access(location, field_start, type_, label, index, record),
+            documentation,
+        } => v.visit_typed_expr_record_access(
+            location,
+            field_start,
+            type_,
+            label,
+            index,
+            record,
+            documentation,
+        ),
         TypedExpr::ModuleSelect {
             location,
             field_start,
@@ -875,7 +910,11 @@ where
             v.visit_typed_expr_negate_bool(location, value)
         }
         TypedExpr::NegateInt { location, value } => v.visit_typed_expr_negate_int(location, value),
-        TypedExpr::Invalid { location, type_ } => v.visit_typed_expr_invalid(location, type_),
+        TypedExpr::Invalid {
+            location,
+            type_,
+            extra_information,
+        } => v.visit_typed_expr_invalid(location, type_, extra_information),
         TypedExpr::Echo {
             location,
             expression,
@@ -968,12 +1007,21 @@ pub fn visit_typed_expr_fn<'a, V>(
     _location: &'a SrcSpan,
     _type_: &'a Arc<Type>,
     _kind: &'a FunctionLiteralKind,
-    _arguments: &'a [TypedArg],
+    arguments: &'a [TypedArg],
     body: &'a Vec1<TypedStatement>,
-    _return_annotation: &'a Option<TypeAst>,
+    return_annotation: &'a Option<TypeAst>,
 ) where
     V: Visit<'a> + ?Sized,
 {
+    for argument in arguments {
+        if let Some(annotation) = &argument.annotation {
+            v.visit_type_ast(annotation);
+        }
+    }
+    if let Some(return_) = return_annotation {
+        v.visit_type_ast(return_);
+    }
+
     for stmt in body {
         v.visit_typed_statement(stmt);
     }
@@ -1046,6 +1094,7 @@ pub fn visit_typed_expr_case<'a, V>(
     }
 }
 
+#[allow(clippy::too_many_arguments)]
 pub fn visit_typed_expr_record_access<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
@@ -1054,6 +1103,7 @@ pub fn visit_typed_expr_record_access<'a, V>(
     _label: &'a EcoString,
     _index: &'a u64,
     record: &'a TypedExpr,
+    _documentation: &'a Option<EcoString>,
 ) where
     V: Visit<'a> + ?Sized,
 {
@@ -1207,6 +1257,9 @@ pub fn visit_typed_assignment<'a, V>(v: &mut V, assignment: &'a TypedAssignment)
 where
     V: Visit<'a> + ?Sized,
 {
+    if let Some(annotation) = &assignment.annotation {
+        v.visit_type_ast(annotation);
+    }
     v.visit_typed_expr(&assignment.value);
     v.visit_typed_pattern(&assignment.pattern);
 }
@@ -1367,6 +1420,7 @@ where
             v.visit_typed_clause_guard(left);
             v.visit_typed_clause_guard(right);
         }
+        super::ClauseGuard::Block { location: _, value } => v.visit_typed_clause_guard(value),
         super::ClauseGuard::Not {
             location: _,
             expression,
@@ -1384,12 +1438,14 @@ where
             tuple,
         } => v.visit_typed_clause_guard_tuple_index(location, index, type_, tuple),
         super::ClauseGuard::FieldAccess {
-            location,
+            label_location,
             index,
             label,
             type_,
             container,
-        } => v.visit_typed_clause_guard_field_access(location, index, label, type_, container),
+        } => {
+            v.visit_typed_clause_guard_field_access(label_location, index, label, type_, container)
+        }
         super::ClauseGuard::ModuleSelect {
             location,
             type_,
@@ -1434,7 +1490,7 @@ pub fn visit_typed_clause_guard_tuple_index<'a, V>(
 
 pub fn visit_typed_clause_guard_field_access<'a, V>(
     v: &mut V,
-    _location: &'a SrcSpan,
+    _label_location: &'a SrcSpan,
     _index: &'a Option<u64>,
     _label: &'a EcoString,
     _type_: &'a Arc<Type>,
@@ -1798,8 +1854,12 @@ where
 {
 }
 
-pub fn visit_typed_expr_invalid<'a, V>(_v: &mut V, _location: &'a SrcSpan, _type_: &'a Arc<Type>)
-where
+pub fn visit_typed_expr_invalid<'a, V>(
+    _v: &mut V,
+    _location: &'a SrcSpan,
+    _type_: &'a Arc<Type>,
+    _extra_information: &'a Option<InvalidExpression>,
+) where
     V: Visit<'a> + ?Sized,
 {
 }
