@@ -272,26 +272,26 @@ impl Module {
 
         self.ast.type_info.documentation = self.ast.documentation.clone();
 
-        // Order statements to avoid misassociating doc comments after the
+        // Order definitions to avoid misassociating doc comments after the
         // order has changed during compilation.
-        let mut statements: Vec<_> = self.ast.definitions.iter_mut().collect();
-        statements.sort_by(|a, b| a.location().start.cmp(&b.location().start));
+        let mut definitions: Vec<_> = self.ast.definitions.iter_mut().collect();
+        definitions.sort_by(|a, b| a.location().start.cmp(&b.location().start));
 
         // Doc Comments
         let mut doc_comments = self.extra.doc_comments.iter().peekable();
-        for statement in &mut statements {
+        for definition in &mut definitions {
             let (docs_start, docs): (u32, Vec<&str>) = doc_comments_before(
                 &mut doc_comments,
                 &self.extra,
-                statement.location().start,
+                definition.location().start,
                 &self.code,
             );
             if !docs.is_empty() {
                 let doc = docs.join("\n").into();
-                statement.put_doc((docs_start, doc));
+                definition.put_doc((docs_start, doc));
             }
 
-            if let Definition::CustomType(CustomType { constructors, .. }) = statement {
+            if let Definition::CustomType(CustomType { constructors, .. }) = definition {
                 for constructor in constructors {
                     let (docs_start, docs): (u32, Vec<&str>) = doc_comments_before(
                         &mut doc_comments,
@@ -490,7 +490,10 @@ fn type_to_definition_locations<'a>(
         // `Wobble`.
         //
         Type::Named {
-            module, name, args, ..
+            module,
+            name,
+            arguments,
+            ..
         } => {
             let Some(module) = importable_modules.get(module) else {
                 return vec![];
@@ -504,9 +507,9 @@ fn type_to_definition_locations<'a>(
                 module: Some(module.name.clone()),
                 span: type_.origin,
             }];
-            for arg in args {
+            for argument in arguments {
                 locations.extend(type_to_definition_locations(
-                    arg.clone(),
+                    argument.clone(),
                     importable_modules,
                 ));
             }
@@ -516,9 +519,9 @@ fn type_to_definition_locations<'a>(
         // For fn types we just get the locations of their arguments and return
         // type.
         //
-        Type::Fn { args, return_ } => args
+        Type::Fn { arguments, return_ } => arguments
             .iter()
-            .flat_map(|arg| type_to_definition_locations(arg.clone(), importable_modules))
+            .flat_map(|argument| type_to_definition_locations(argument.clone(), importable_modules))
             .chain(type_to_definition_locations(
                 return_.clone(),
                 importable_modules,
