@@ -101,10 +101,14 @@ pub fn generate_html<IO: FileSystemReader>(
         path: doc_link.href.to_string(),
     });
 
-    let repo_link = config.repository.url().map(|path| Link {
-        name: "Repository".into(),
-        path,
-    });
+    let repo_link = config
+        .repository
+        .as_ref()
+        .map(|r| r.url())
+        .map(|path| Link {
+            name: "Repository".into(),
+            path,
+        });
 
     let host = if is_hex_publish == DocContext::HexPublish {
         "https://hexdocs.pm"
@@ -212,7 +216,7 @@ pub fn generate_html<IO: FileSystemReader>(
             .ast
             .definitions
             .iter()
-            .filter_map(|statement| printer.type_definition(&source_links, statement))
+            .filter_map(|definition| printer.type_definition(&source_links, definition))
             .sorted()
             .collect();
 
@@ -220,7 +224,7 @@ pub fn generate_html<IO: FileSystemReader>(
             .ast
             .definitions
             .iter()
-            .filter_map(|statement| printer.value(&source_links, statement))
+            .filter_map(|definition| printer.value(&source_links, definition))
             .sorted()
             .collect();
 
