@@ -1123,6 +1123,69 @@ pub type External =
     );
 }
 
+#[test]
+fn use_reexport_from_other_package() {
+    assert_documentation!(
+        ("some_package", "some_package/internal", "pub type Internal"),
+        (
+            "some_package",
+            "some_package/api",
+            "
+import some_package/internal
+pub type External = internal.Internal
+"
+        ),
+        "
+import some_package/api
+
+pub fn do_thing(value: api.External) {
+  value
+}
+",
+        ONLY_LINKS
+    );
+}
+
+#[test]
+fn function_uses_reexport_of_internal_type() {
+    assert_documentation!(
+        ("thepackage/internal", "pub type Internal"),
+        "
+import thepackage/internal
+
+pub type External = internal.Internal
+
+pub fn do_thing(value: internal.Internal) -> External {
+  value
+}
+",
+        ONLY_LINKS
+    );
+}
+
+#[test]
+fn function_uses_reexport_of_internal_type_in_other_module() {
+    assert_documentation!(
+        ("thepackage/internal", "pub type Internal"),
+        (
+            "thepackage/something",
+            "
+import thepackage/internal
+
+pub type External = internal.Internal
+"
+        ),
+        "
+import thepackage/something
+
+pub fn do_thing(value: something.External) {
+  value
+}
+",
+        ONLY_LINKS
+    );
+}
+
 #[test]
 fn constructor_with_long_types_and_many_fields() {
     assert_documentation!(
@@ -1190,3 +1253,47 @@ pub fn aaaaaaaaaaaaaaaaaaaaaaaaaaaa() -> aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa {
         NONE
     );
 }
+
+#[test]
+fn forgejo_single_line_definition() {
+    let mut config = PackageConfig::default();
+    let repo = Repository::Forgejo {
+        host: "https://code.example.org/".parse::<Uri>().unwrap(),
+        user: "wibble".into(),
+        repo: "wobble".into(),
+        path: None,
+        tag_prefix: None,
+    };
+
+    config.name = EcoString::from("test_project_name");
+    config.repository = Some(repo);
+
+    let modules = vec![("app.gleam", "pub type Wibble = Int")];
+    let html = compile(config, modules);
+
+    assert!(
+        html.contains("https://code.example.org/wibble/wobble/src/tag/v0.1.0/src/app.gleam#L1")
+    );
+}
+
+#[test]
+fn forgejo_multiple_line_definition() {
+    let mut config = PackageConfig::default();
+    let repo = Repository::Forgejo {
+        host: "https://code.example.org/".parse::<Uri>().unwrap(),
+        user: "wibble".into(),
+        repo: "wobble".into(),
+        path: None,
+        tag_prefix: None,
+    };
+
+    config.name = EcoString::from("test_project_name");
+    config.repository = Some(repo);
+
+    let modules = vec![("app.gleam", "pub type Wibble \n\n= Int")];
+    let html = compile(config, modules);
+
+    assert!(
+        html.contains("https://code.example.org/wibble/wobble/src/tag/v0.1.0/src/app.gleam#L1-L3")
+    );
+}
