@@ -3,10 +3,10 @@ use std::time::SystemTime;
 use camino::Utf8PathBuf;
 
 use crate::analyse::TargetSupport;
-use crate::build;
 use crate::config::PackageConfig;
 use crate::type_::PRELUDE_MODULE_NAME;
 use crate::warning::WarningEmitter;
+use crate::{build, inline};
 use crate::{
     build::{Origin, Target},
     erlang::module,
@@ -27,6 +27,7 @@ mod echo;
 mod external_fn;
 mod functions;
 mod guards;
+mod inlining;
 mod let_assert;
 mod numbers;
 mod panic;
@@ -40,7 +41,11 @@ mod type_params;
 mod use_;
 mod variables;
 
-pub fn compile_test_project(src: &str, src_path: &str, dep: Option<(&str, &str, &str)>) -> String {
+pub fn compile_test_project(
+    src: &str,
+    src_path: &str,
+    dependencies: Vec<(&str, &str, &str)>,
+) -> String {
     let mut modules = im::HashMap::new();
     let ids = UniqueIdGenerator::new();
     // DUPE: preludeinsertion
@@ -52,7 +57,7 @@ pub fn compile_test_project(src: &str, src_path: &str, dep: Option<(&str, &str,
         crate::type_::build_prelude(&ids),
     );
     let mut direct_dependencies = std::collections::HashMap::from_iter(vec![]);
-    if let Some((dep_package, dep_name, dep_src)) = dep {
+    for (dep_package, dep_name, dep_src) in dependencies {
         let mut dep_config = PackageConfig::default();
         dep_config.name = dep_package.into();
         let parsed = crate::parse::parse_module(
@@ -72,6 +77,7 @@ pub fn compile_test_project(src: &str, src_path: &str, dep: Option<(&str, &str,
             importable_modules: &modules,
             warnings: &TypeWarningEmitter::null(),
             direct_dependencies: &std::collections::HashMap::new(),
+            dev_dependencies: &std::collections::HashSet::new(),
             target_support: TargetSupport::NotEnforced,
             package_config: &dep_config,
         }
@@ -95,12 +101,15 @@ pub fn compile_test_project(src: &str, src_path: &str, dep: Option<(&str, &str,
         importable_modules: &modules,
         warnings: &TypeWarningEmitter::null(),
         direct_dependencies: &direct_dependencies,
+        dev_dependencies: &std::collections::HashSet::new(),
         target_support: TargetSupport::NotEnforced,
         package_config: &config,
     }
     .infer_module(ast, line_numbers, path.clone())
     .expect("should successfully infer root Erlang");
 
+    let ast = inline::module(ast, &modules);
+
     // After building everything we still need to attach the module comments, to
     // do that we're reusing the `attach_doc_and_module_comments` that's used
     // for the real thing. We just have to make a placeholder module wrapping
@@ -129,11 +138,11 @@ pub fn compile_test_project(src: &str, src_path: &str, dep: Option<(&str, &str,
 
 #[macro_export]
 macro_rules! assert_erl {
-    (($dep_package:expr, $dep_name:expr, $dep_src:expr), $src:expr $(,)?) => {{
+    ($(($dep_package:expr, $dep_name:expr, $dep_src:expr)),+, $src:literal $(,)?) => {{
         let compiled = $crate::erlang::tests::compile_test_project(
             $src,
             "/root/project/test/my/mod.gleam",
-            Some(($dep_package, $dep_name, $dep_src)),
+            vec![$(($dep_package, $dep_name, $dep_src)),*],
         );
         let output = format!(
             "----- SOURCE CODE\n{}\n\n----- COMPILED ERLANG\n{}",
@@ -146,11 +155,11 @@ macro_rules! assert_erl {
         let compiled = $crate::erlang::tests::compile_test_project(
             $src,
             "/root/project/test/my/mod.gleam",
-            None,
+            Vec::new(),
         );
         let output = format!(
             "----- SOURCE CODE\n{}\n\n----- COMPILED ERLANG\n{}",
-            $src, compiled
+            $src, compiled,
         );
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     }};
@@ -985,7 +994,7 @@ pub fn main() {
 fn windows_file_escaping_bug() {
     let src = "pub fn main() { Nil }";
     let path = "C:\\root\\project\\test\\my\\mod.gleam";
-    let output = compile_test_project(src, path, None);
+    let output = compile_test_project(src, path, Vec::new());
     insta::assert_snapshot!(insta::internals::AutoName, output, src);
 }
 
