@@ -4653,26 +4653,15 @@ fn do_not_remove_required_braces_case_guard() {
 }
 
 #[test]
-fn remove_braces_case_guard() {
-    assert_format_rewrite!(
-        "fn main() {
-  let is_enabled = False
-  let is_confirmed = False
-  let is_admin = True
-  case is_enabled, is_confirmed, is_admin {
-    is_enabled, is_confirmed, is_admin if { is_enabled && is_confirmed } || is_admin ->
-      Nil
-    _, _, _ -> Nil
-  }
-}
-",
+fn do_not_remove_braces_from_case_guard() {
+    assert_format!(
         "fn main() {
   let is_enabled = False
   let is_confirmed = False
   let is_admin = True
   case is_enabled, is_confirmed, is_admin {
     is_enabled, is_confirmed, is_admin
-      if is_enabled && is_confirmed || is_admin
+      if { is_enabled && is_confirmed } || is_admin
     -> Nil
     _, _, _ -> Nil
   }
@@ -4682,8 +4671,8 @@ fn remove_braces_case_guard() {
 }
 
 #[test]
-fn remove_braces_case_guard_2() {
-    assert_format_rewrite!(
+fn do_not_remove_braces_from_case_guard_2() {
+    assert_format!(
         "fn main() {
   let wibble = #(10, [0])
   case wibble {
@@ -4691,14 +4680,6 @@ fn remove_braces_case_guard_2() {
     _ -> Nil
   }
 }
-",
-        "fn main() {
-  let wibble = #(10, [0])
-  case wibble {
-    wibble if True && wibble.0 == 10 -> Nil
-    _ -> Nil
-  }
-}
 "
     );
 }
@@ -4902,7 +4883,7 @@ fn double_negate() {
 "#,
         r#"pub fn main() {
   let a = 3
-  let b = - -a
+  let b = a
 }
 "#
     );
@@ -4918,7 +4899,7 @@ fn triple_negate() {
 "#,
         r#"pub fn main() {
   let a = 3
-  let b = - - -a
+  let b = -a
 }
 "#
     );
@@ -4950,14 +4931,32 @@ fn binary_double_negate() {
 "#,
         r#"pub fn main() {
   let a = 3
-  let b = - -{ a + 3 }
+  let b = { a + 3 }
 }
 "#
     );
 }
 
 #[test]
-fn repeated_negate_after_subtract() {
+fn even_repeated_negate_after_subtract() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  let a = 3
+  let b = 4
+  let c = a-------b
+}
+"#,
+        r#"pub fn main() {
+  let a = 3
+  let b = 4
+  let c = a - b
+}
+"#
+    );
+}
+
+#[test]
+fn odd_repeated_negate_after_subtract() {
     assert_format_rewrite!(
         r#"pub fn main() {
   let a = 3
@@ -4968,12 +4967,26 @@ fn repeated_negate_after_subtract() {
         r#"pub fn main() {
   let a = 3
   let b = 4
-  let c = a - - - - - - - -b
+  let c = a - -b
 }
 "#
     );
 }
 
+#[test]
+fn double_negation_on_bools_is_removed() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  !!True
+}
+"#,
+        "pub fn main() {
+  True
+}
+"
+    );
+}
+
 #[test]
 fn wrap_long_line_with_int_negation() {
     assert_format_rewrite!(
@@ -6685,3 +6698,38 @@ fn assert_as_with_comment_before_the_as() {
 "#
     );
 }
+
+// https://github.com/gleam-lang/gleam/issues/4664
+#[test]
+fn pattern_unused_discard() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  let a = 10
+  let _ = case a {
+    _ as b -> b
+  }
+}
+"#,
+        r#"pub fn main() {
+  let a = 10
+  let _ = case a {
+    b -> b
+  }
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/4929
+#[test]
+fn format_panic_as_with_block_message() {
+    assert_format!(
+        r#"pub fn main() {
+  panic as {
+    // b
+    a
+  }
+}
+"#
+    );
+}
