@@ -430,7 +430,10 @@ pub trait TarUnpacker {
 
 #[inline]
 pub fn is_native_file_extension(extension: &str) -> bool {
-    matches!(extension, "erl" | "hrl" | "ex" | "js" | "mjs" | "ts")
+    matches!(
+        extension,
+        "erl" | "hrl" | "ex" | "js" | "mjs" | "cjs" | "ts"
+    )
 }
 
 pub fn ordered_map<S, K, V>(value: &HashMap<K, V>, serializer: S) -> Result<S::Ok, S::Error>
