@@ -5,13 +5,15 @@ mod import;
 mod tests;
 mod typescript;
 
+use std::collections::HashMap;
+
 use num_bigint::BigInt;
 use num_traits::ToPrimitive;
 
 use crate::build::Target;
 use crate::build::package_compiler::StdlibPackage;
 use crate::codegen::TypeScriptDeclarations;
-use crate::type_::PRELUDE_MODULE_NAME;
+use crate::type_::{PRELUDE_MODULE_NAME, RecordAccessor};
 use crate::{
     ast::{CustomType, Function, Import, ModuleConstant, TypeAlias, *},
     docvec,
@@ -340,6 +342,7 @@ impl<'a> Generator<'a> {
 
     fn custom_type_definition(
         &mut self,
+        name: &'a str,
         constructors: &'a [TypedRecordConstructor],
         publicity: Publicity,
         opaque: bool,
@@ -350,17 +353,201 @@ impl<'a> Generator<'a> {
         }
 
         self.tracker.custom_type_used = true;
-        constructors
+
+        let constructor_publicity = if opaque || publicity.is_private() {
+            Publicity::Private
+        } else {
+            Publicity::Public
+        };
+
+        let mut definitions = constructors
             .iter()
-            .map(|constructor| self.record_definition(constructor, publicity, opaque))
-            .collect()
+            .map(|constructor| self.variant_definition(constructor, name, constructor_publicity))
+            .collect_vec();
+
+        // Generate getters for fields shared between variants
+        if let Some(accessors_map) = self.module.type_info.accessors.get(name)
+            && !accessors_map.shared_accessors.is_empty()
+            // Don't bother generating shared getters when there's only one variant,
+            // since the specific accessors can always be uses instead.
+            && constructors.len() != 1
+            // Only generate accessors for the API if the constructors are public
+            && constructor_publicity.is_public()
+        {
+            definitions.push(self.shared_custom_type_fields(name, &accessors_map.shared_accessors));
+        }
+
+        definitions
     }
 
-    fn record_definition(
+    fn variant_definition(
+        &self,
+        constructor: &'a TypedRecordConstructor,
+        type_name: &'a str,
+        publicity: Publicity,
+    ) -> Document<'a> {
+        let class_definition = self.variant_class_definition(constructor, publicity);
+
+        // If the custom type is private or opaque, we don't need to generate API
+        // functions for it.
+        if publicity.is_private() {
+            return class_definition;
+        }
+
+        let constructor_definition = self.variant_constructor_definition(constructor, type_name);
+        let variant_check_definition = self.variant_check_definition(constructor, type_name);
+        let fields_definition = self.variant_fields_definition(constructor, type_name);
+
+        docvec![
+            class_definition,
+            line(),
+            constructor_definition,
+            line(),
+            variant_check_definition,
+            fields_definition,
+        ]
+    }
+
+    fn variant_constructor_definition(
+        &self,
+        constructor: &'a TypedRecordConstructor,
+        type_name: &'a str,
+    ) -> Document<'a> {
+        let mut arguments = Vec::new();
+
+        for (index, parameter) in constructor.arguments.iter().enumerate() {
+            if let Some((_, label)) = &parameter.label {
+                arguments.push(maybe_escape_identifier(label).to_doc());
+            } else {
+                arguments.push(eco_format!("${index}").to_doc());
+            }
+        }
+
+        let construction = docvec![
+            break_("", " "),
+            "new ",
+            constructor.name.as_str(),
+            "(",
+            join(arguments.clone(), break_(",", ", ")).group(),
+            ");"
+        ]
+        .group();
+
+        docvec![
+            "export const ",
+            type_name,
+            "$",
+            constructor.name.as_str(),
+            " = (",
+            join(arguments, break_(",", ", ")),
+            ") =>",
+            construction.nest(INDENT),
+        ]
+    }
+
+    fn variant_check_definition(
+        &self,
+        constructor: &'a TypedRecordConstructor,
+        type_name: &'a str,
+    ) -> Document<'a> {
+        let construction = docvec![
+            break_("", " "),
+            "value instanceof ",
+            constructor.name.as_str(),
+            ";"
+        ]
+        .group();
+
+        docvec![
+            "export const ",
+            type_name,
+            "$is",
+            constructor.name.as_str(),
+            " = (value) =>",
+            construction.nest(INDENT),
+        ]
+    }
+
+    fn variant_fields_definition(
+        &self,
+        constructor: &'a TypedRecordConstructor,
+        type_name: &'a str,
+    ) -> Document<'a> {
+        let mut functions = Vec::new();
+
+        for (index, argument) in constructor.arguments.iter().enumerate() {
+            // Always generate the accessor for the value at this index. Although
+            // this is not necessary when a label is present, we want to make sure
+            // that adding a label to a record isn't a breaking change. For this
+            // reason, we need to generate an index getter even when a label is
+            // present to ensure consistent behaviour between labelled and unlabelled
+            // field access.
+            let function_name = eco_format!(
+                "{type_name}${record_name}${index}",
+                record_name = constructor.name,
+            );
+
+            let contents;
+
+            // If the argument is labelled, also generate a getter for the labelled
+            // argument.
+            if let Some((_, label)) = &argument.label {
+                let function_name = eco_format!(
+                    "{type_name}${record_name}${label}",
+                    record_name = constructor.name,
+                );
+
+                contents =
+                    docvec![break_("", " "), "value.", maybe_escape_property(label), ";"].group();
+
+                functions.push(docvec![
+                    line(),
+                    "export const ",
+                    function_name,
+                    " = (value) =>",
+                    contents.clone().nest(INDENT),
+                ]);
+            } else {
+                contents = docvec![break_("", " "), "value[", index, "];"].group()
+            }
+
+            functions.push(docvec![
+                line(),
+                "export const ",
+                function_name,
+                " = (value) =>",
+                contents.nest(INDENT),
+            ]);
+        }
+
+        concat(functions)
+    }
+
+    fn shared_custom_type_fields(
+        &self,
+        type_name: &'a str,
+        shared_accessors: &HashMap<EcoString, RecordAccessor>,
+    ) -> Document<'a> {
+        let accessors = shared_accessors.keys().sorted().map(|field| {
+            let function_name = eco_format!("{type_name}${field}");
+
+            let contents =
+                docvec![break_("", " "), "value.", maybe_escape_property(field), ";"].group();
+
+            docvec![
+                "export const ",
+                function_name,
+                " = (value) =>",
+                contents.nest(INDENT),
+            ]
+        });
+        concat(Itertools::intersperse(accessors, line()))
+    }
+
+    fn variant_class_definition(
         &self,
         constructor: &'a TypedRecordConstructor,
         publicity: Publicity,
-        opaque: bool,
     ) -> Document<'a> {
         fn parameter((i, arg): (usize, &TypedRecordConstructorArg)) -> Document<'_> {
             arg.label
@@ -376,10 +563,10 @@ impl<'a> Generator<'a> {
             nil()
         };
 
-        let head = if publicity.is_private() || opaque {
-            "class "
-        } else {
+        let head = if publicity.is_public() {
             "export class "
+        } else {
+            "class "
         };
         let head = docvec![head, &constructor.name, " extends $CustomType {"];
 
@@ -438,8 +625,9 @@ impl<'a> Generator<'a> {
                     publicity,
                     constructors,
                     opaque,
+                    name,
                     ..
-                }) => self.custom_type_definition(constructors, *publicity, *opaque),
+                }) => self.custom_type_definition(name, constructors, *publicity, *opaque),
 
                 Definition::Function(Function { .. })
                 | Definition::TypeAlias(TypeAlias { .. })
@@ -647,7 +835,7 @@ impl<'a> Generator<'a> {
             "export function "
         };
 
-        let body = generator.function_body(&function.body, function.arguments.as_slice());
+        let body = generator.function_body(function.body.as_slice(), function.arguments.as_slice());
 
         docvec![
             function_doc,
