@@ -8,7 +8,7 @@ use crate::{
     exhaustiveness::{
         BitArrayMatchedValue, BitArrayTest, Body, BoundValue, CompiledCase, Decision,
         FallbackCheck, MatchTest, Offset, ReadAction, ReadSize, ReadType, RuntimeCheck,
-        SizeOperator, SizeTest, StringEncoding, Variable, VariableUsage,
+        SizeOperator, SizeTest, Variable, VariableUsage,
     },
     format::break_block,
     javascript::{
@@ -16,9 +16,7 @@ use crate::{
         maybe_escape_property,
     },
     pretty::{Document, Documentable, break_, concat, join, line, nil},
-    strings::{
-        convert_string_escape_chars, length_utf16, string_to_utf16_bytes, string_to_utf32_bytes,
-    },
+    strings::{convert_string_escape_chars, length_utf16},
 };
 use ecow::{EcoString, eco_format};
 use itertools::Itertools;
@@ -339,7 +337,7 @@ impl<'a> CasePrinter<'_, '_, 'a, '_> {
     fn switch(
         &mut self,
         var: &'a Variable,
-        choices: &'a [(RuntimeCheck, Box<Decision>)],
+        choices: &'a [(RuntimeCheck, Decision)],
         fallback: &'a Decision,
         fallback_check: &'a FallbackCheck,
     ) -> CaseBody<'a> {
@@ -702,10 +700,10 @@ pub fn let_<'a>(
     // We must generate this after we generate the code for the decision tree
     // itself as we might be re-binding variables which are used in the checks
     // to determine whether the pattern matches or not.
-    let beginning_assignments = pattern.bound_variables().into_iter().map(|variable| {
+    let beginning_assignments = pattern.bound_variables().into_iter().map(|bound_variable| {
         docvec![
             "let ",
-            expression_generator.local_var(&variable),
+            expression_generator.local_var(&bound_variable.name()),
             ";",
             line()
         ]
@@ -718,7 +716,7 @@ pub fn let_<'a>(
     ];
 
     match scope_position {
-        expression::Position::NotTail(_ordering) => doc,
+        expression::Position::Expression(_) | expression::Position::Statement => doc,
         expression::Position::Tail => docvec![doc, line(), "return ", assignment_name, ";"],
         expression::Position::Assign(variable) => {
             docvec![doc, line(), variable, " = ", assignment_name, ";"]
@@ -1083,20 +1081,16 @@ impl<'generator, 'module, 'a> Variables<'generator, 'module, 'a> {
                     read_action,
                 }) => match expected {
                     BitArrayMatchedValue::LiteralString {
-                        value: expected,
-                        encoding,
-                    } => self.literal_string_segment_bytes_check(
-                        value,
-                        expected,
-                        read_action,
-                        *encoding,
-                    ),
+                        value: _,
+                        encoding: _,
+                        bytes: expected,
+                    } => self.literal_string_segment_bytes_check(value, expected, read_action),
                     BitArrayMatchedValue::LiteralFloat(expected) => {
                         self.literal_float_segment_bytes_check(value, expected, read_action)
                     }
-                    BitArrayMatchedValue::LiteralInt(expected) => {
-                        self.literal_int_segment_bytes_check(value, expected.clone(), read_action)
-                    }
+                    BitArrayMatchedValue::LiteralInt {
+                        value: expected, ..
+                    } => self.literal_int_segment_bytes_check(value, expected.clone(), read_action),
                     BitArrayMatchedValue::Variable(..)
                     | BitArrayMatchedValue::Discard(..)
                     | BitArrayMatchedValue::Assign { .. } => {
@@ -1423,9 +1417,9 @@ impl<'generator, 'module, 'a> Variables<'generator, 'module, 'a> {
         &mut self,
         // A string representing the bit array value we read bits from.
         bit_array: EcoString,
-        literal_string: &EcoString,
+        // The bytes of the literal string we should be matching on.
+        string_bytes: &Vec<u8>,
         read_action: &ReadAction,
-        encoding: StringEncoding,
     ) -> Document<'a> {
         let ReadAction {
             from: start,
@@ -1437,22 +1431,7 @@ impl<'generator, 'module, 'a> Variables<'generator, 'module, 'a> {
 
         let equality = " === ";
 
-        let escaped = convert_string_escape_chars(literal_string);
-        // We need to have this vector here so that we don't run into lifetime
-        // issues when calling `.as_slice` on the local vectors created when this
-        // isn't a UTF-8 string.
-        let bytes_vec;
-        let bytes = match encoding {
-            StringEncoding::Utf8 => escaped.as_bytes(),
-            StringEncoding::Utf16 => {
-                bytes_vec = string_to_utf16_bytes(&escaped, read_action.endianness);
-                bytes_vec.as_slice()
-            }
-            StringEncoding::Utf32 => {
-                bytes_vec = string_to_utf32_bytes(&escaped, read_action.endianness);
-                bytes_vec.as_slice()
-            }
-        };
+        let bytes = string_bytes.as_slice();
 
         if let Some(mut from_byte) = start.constant_bytes() {
             // If the string starts at a compile-time known byte, then we can
@@ -1464,14 +1443,18 @@ impl<'generator, 'module, 'a> Variables<'generator, 'module, 'a> {
                 from_byte += 1;
             }
         } else {
+            let mut start = start.clone();
+
             // If the string doesn't start at a byte aligned offset then we'll
             // have to take slices out of it to check that each byte matches.
             for byte in bytes {
-                let end = self.offset_to_doc(&start.add_constant(8), false);
-                let from = self.offset_to_doc(start, false);
-                let byte_access =
-                    self.bit_array_slice_to_int(&bit_array, from, end, endianness, *signed);
+                let start_doc = self.offset_to_doc(&start, false);
+                let end = start.add_constant(8);
+                let end_doc = self.offset_to_doc(&end, false);
+                let byte_access = self
+                    .bit_array_slice_to_int(&bit_array, start_doc, end_doc, endianness, *signed);
                 checks.push(docvec![byte_access, equality, byte]);
+                start = end;
             }
         }
 
