@@ -2,6 +2,7 @@ use crate::{
     analyse::TargetSupport,
     build::{Origin, Target},
     config::PackageConfig,
+    inline,
     javascript::*,
     uid::UniqueIdGenerator,
     warning::{TypeWarningEmitter, WarningEmitter},
@@ -21,6 +22,7 @@ mod echo;
 mod externals;
 mod functions;
 mod generics;
+mod inlining;
 mod lists;
 mod modules;
 mod numbers;
@@ -50,9 +52,9 @@ macro_rules! assert_js {
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 
-    (($dep_package:expr, $dep_name:expr, $dep_src:expr), $src:expr $(,)?) => {{
+    ($(($dep_package:expr, $dep_name:expr, $dep_src:expr)),+, $src:literal $(,)?) => {{
         let compiled =
-            $crate::javascript::tests::compile_js($src, vec![($dep_package, $dep_name, $dep_src)]);
+            $crate::javascript::tests::compile_js($src, vec![$(($dep_package, $dep_name, $dep_src)),*]);
         let output = format!(
             "----- SOURCE CODE\n{}\n\n----- COMPILED JAVASCRIPT\n{}",
             $src, compiled
@@ -131,7 +133,7 @@ pub fn compile(src: &str, deps: Vec<(&str, &str, &str)>) -> TypedModule {
         PRELUDE_MODULE_NAME.into(),
         crate::type_::build_prelude(&ids),
     );
-    let mut direct_dependencies = std::collections::HashMap::from_iter(vec![]);
+    let mut direct_dependencies = HashMap::from_iter(vec![]);
 
     deps.iter().for_each(|(dep_package, dep_name, dep_src)| {
         let mut dep_config = PackageConfig::default();
@@ -152,7 +154,8 @@ pub fn compile(src: &str, deps: Vec<(&str, &str, &str)>) -> TypedModule {
             origin: Origin::Src,
             importable_modules: &modules,
             warnings: &TypeWarningEmitter::null(),
-            direct_dependencies: &std::collections::HashMap::new(),
+            direct_dependencies: &HashMap::new(),
+            dev_dependencies: &std::collections::HashSet::new(),
             target_support: TargetSupport::Enforced,
             package_config: &dep_config,
         }
@@ -171,18 +174,21 @@ pub fn compile(src: &str, deps: Vec<(&str, &str, &str)>) -> TypedModule {
     let mut config = PackageConfig::default();
     config.name = "thepackage".into();
 
-    crate::analyse::ModuleAnalyzerConstructor::<()> {
+    let module = crate::analyse::ModuleAnalyzerConstructor::<()> {
         target: Target::JavaScript,
         ids: &ids,
         origin: Origin::Src,
         importable_modules: &modules,
         warnings: &TypeWarningEmitter::null(),
         direct_dependencies: &direct_dependencies,
+        dev_dependencies: &std::collections::HashSet::new(),
         target_support: TargetSupport::NotEnforced,
         package_config: &config,
     }
     .infer_module(ast, line_numbers, "src/module.gleam".into())
-    .expect("should successfully infer")
+    .expect("should successfully infer");
+
+    inline::module(module, &modules)
 }
 
 pub fn compile_js(src: &str, deps: Vec<(&str, &str, &str)>) -> String {
