@@ -4,7 +4,7 @@ use crate::{assert_js, assert_ts_def};
 fn tuple_matching() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert #(1, 2) = x
 }
 "#,
@@ -13,19 +13,19 @@ fn go(x) {
 
 #[test]
 fn assert() {
-    assert_js!(r#"fn go(x) { let assert 1 = x }"#,);
+    assert_js!(r#"pub fn go(x) { let assert 1 = x }"#,);
 }
 
 #[test]
 fn assert1() {
-    assert_js!(r#"fn go(x) { let assert #(1, 2) = x }"#,);
+    assert_js!(r#"pub fn go(x) { let assert #(1, 2) = x }"#,);
 }
 
 #[test]
 fn nested_binding() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert #(a, #(b, c, 2) as t, _, 1) = x
 }
 "#,
@@ -37,7 +37,7 @@ fn variable_renaming() {
     assert_js!(
         r#"
 
-fn go(x, wibble) {
+pub fn go(x, wibble) {
   let a = 1
   wibble(a)
   let a = 2
@@ -64,7 +64,7 @@ fn constant_assignments() {
         r#"
 const a = True
 
-fn go() {
+pub fn go() {
   a
   let a = 10
   a + 20
@@ -80,7 +80,7 @@ fn second() {
 
 #[test]
 fn returning_literal_subject() {
-    assert_js!(r#"fn go(x) { let assert 1 = x + 1 }"#,);
+    assert_js!(r#"pub fn go(x) { let assert 1 = x + 1 }"#,);
 }
 
 #[test]
@@ -274,7 +274,7 @@ type Wibble {
     Wibble(Int)
 }
 
-fn go() {
+pub fn go() {
   let assert Wibble(n) = Wibble(1)
   n
 }
@@ -291,7 +291,7 @@ type Wibble {
     Wobble(Int)
 }
 
-fn go() {
+pub fn go() {
   let assert Wobble(n) = Wibble(1)
   n
 }
@@ -308,7 +308,7 @@ type Wibble {
     Wobble(Int)
 }
 
-fn go() {
+pub fn go() {
   let assert _ = Wibble(1)
   1
 }
@@ -326,7 +326,7 @@ type Wibble {
     Woo(Int)
 }
 
-fn go() {
+pub fn go() {
   let assert Wobble(n) = todo
   n
 }
@@ -346,7 +346,7 @@ type Wibble {
 
 fn fun(f) { f(Wibble(1)) }
 
-fn go() {
+pub fn go() {
   use _ <- fun
   1
 }
@@ -360,7 +360,7 @@ fn use_matching_assignment() {
         r#"
 fn fun(f) { f(#(2, 4)) }
 
-fn go() {
+pub fn go() {
   use #(_, n) <- fun
   n
 }
