@@ -4,7 +4,7 @@ use crate::{assert_js, assert_ts_def};
 fn tuple() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   #("1", "2", "3")
 }
 "#,
@@ -15,7 +15,7 @@ fn go() {
 fn tuple1() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   #(
     "1111111111111111111111111111111",
     #("1111111111111111111111111111111", "2", "3"),
@@ -41,7 +41,7 @@ pub fn go() {
 fn tuple_access() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   #(1, 2).0
 }
 "#,
@@ -52,7 +52,7 @@ fn go() {
 fn tuple_with_block_element() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   #(
     "1",
     {
@@ -69,7 +69,7 @@ fn go() {
 fn tuple_with_block_element1() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   #(
     "1111111111111111111111111111111",
     #("1111111111111111111111111111111", "2", "3"),
@@ -84,10 +84,10 @@ fn go() {
 fn constant_tuples() {
     assert_js!(
         r#"
-const a = "Hello"
-const b = 1
-const c = 2.0
-const e = #("bob", "dug")
+pub const a = "Hello"
+pub const b = 1
+pub const c = 2.0
+pub const e = #("bob", "dug")
         "#,
     );
 }
@@ -96,7 +96,7 @@ const e = #("bob", "dug")
 fn constant_tuples1() {
     assert_js!(
         r#"
-const e = #(
+pub const e = #(
   "loooooooooooooong", "loooooooooooong", "loooooooooooooong",
   "loooooooooooooong", "loooooooooooong", "loooooooooooooong",
 )
@@ -121,7 +121,7 @@ pub const e = #(
 fn case() {
     assert_js!(
         r#"
-fn go(a) {
+pub fn go(a) {
   case a {
     #(2, a) -> a
     #(1, 1) -> 1
@@ -136,7 +136,7 @@ fn go(a) {
 fn nested_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     #(2, #(a, b)) -> a + b
     _ -> 1
