@@ -17,7 +17,6 @@ use crate::{
         TypedExpr, visit::Visit,
     },
     build::{Module, Origin},
-    io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     line_numbers::LineNumbers,
     type_::{
         self, FieldMap, ModuleInterface, PRELUDE_MODULE_NAME, PreludeType, RecordAccessor, Type,
@@ -27,7 +26,6 @@ use crate::{
 };
 
 use super::{
-    DownloadDependencies, MakeLocker,
     compiler::LspProjectCompiler,
     edits::{
         Newlines, add_newlines_after_import, get_import, get_import_edit,
@@ -95,17 +93,7 @@ pub struct Completer<'a, IO> {
     pub module_line_numbers: LineNumbers,
 }
 
-impl<'a, IO> Completer<'a, IO>
-where
-    // IO to be supplied from outside of gleam-core
-    IO: FileSystemReader
-        + FileSystemWriter
-        + BeamCompiler
-        + CommandExecutor
-        + DownloadDependencies
-        + MakeLocker
-        + Clone,
-{
+impl<'a, IO> Completer<'a, IO> {
     pub fn new(
         src: &'a EcoString,
         params: &'a TextDocumentPositionParams,
@@ -455,10 +443,10 @@ where
                 if let Some(module) = import.used_name() {
                     // If the user has already started a module select then don't show irrelevant modules.
                     // e.x. when the user has typed mymodule.| we should only show items from mymodule.
-                    if let Some(input_mod_name) = &module_select {
-                        if &module != input_mod_name {
-                            continue;
-                        }
+                    if let Some(input_mod_name) = &module_select
+                        && &module != input_mod_name
+                    {
+                        continue;
                     }
                     completions.push(type_completion(
                         Some(&module),
@@ -515,10 +503,10 @@ where
 
             // If the user has already started a module select then don't show irrelevant modules.
             // e.x. when the user has typed mymodule.| we should only show items from mymodule.
-            if let Some(input_mod_name) = &module_select {
-                if qualifier != input_mod_name {
-                    continue;
-                }
+            if let Some(input_mod_name) = &module_select
+                && qualifier != input_mod_name
+            {
+                continue;
             }
 
             // Qualified types
@@ -650,10 +638,10 @@ where
                 if let Some(module) = import.used_name() {
                     // If the user has already started a module select then don't show irrelevant modules.
                     // e.x. when the user has typed mymodule.| we should only show items from mymodule.
-                    if let Some(input_mod_name) = &module_select {
-                        if &module != input_mod_name {
-                            continue;
-                        }
+                    if let Some(input_mod_name) = &module_select
+                        && &module != input_mod_name
+                    {
+                        continue;
                     }
                     completions.push(value_completion(
                         Some(&module),
@@ -709,10 +697,10 @@ where
 
             // If the user has already started a module select then don't show irrelevant modules.
             // e.x. when the user has typed mymodule.| we should only show items from mymodule.
-            if let Some(input_mod_name) = &module_select {
-                if qualifier != input_mod_name {
-                    continue;
-                }
+            if let Some(input_mod_name) = &module_select
+                && qualifier != input_mod_name
+            {
+                continue;
             }
 
             // Qualified values
