@@ -12,8 +12,14 @@ use crate::{
     config::PackageConfig,
     io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     language_server::{
-        code_action::RemoveBlock, compiler::LspProjectCompiler, files::FileSystemProxy,
+        code_action::{
+            AddOmittedLabels, CollapseNestedCase, ExtractFunction, RemoveBlock,
+            RemovePrivateOpaque, RemoveUnreachableCaseClauses,
+        },
+        compiler::LspProjectCompiler,
+        files::FileSystemProxy,
         progress::ProgressReporter,
+        reference::FindVariableReferences,
     },
     line_numbers::LineNumbers,
     paths::ProjectPaths,
@@ -51,8 +57,7 @@ use super::{
     },
     completer::Completer,
     reference::{
-        Referenced, VariableReferenceKind, find_module_references, find_variable_references,
-        reference_for_ast_node,
+        Referenced, VariableReferenceKind, find_module_references, reference_for_ast_node,
     },
     rename::{RenameTarget, Renamed, rename_local_variable, rename_module_entity},
     signature_help, src_span_to_lsp_range,
@@ -389,6 +394,7 @@ where
             actions.extend(RemoveUnusedImports::new(module, &lines, &params).code_actions());
             code_action_convert_qualified_constructor_to_unqualified(
                 module,
+                &this.compiler,
                 &lines,
                 &params,
                 &mut actions,
@@ -402,6 +408,9 @@ where
             code_action_fix_names(&lines, &params, &this.error, &mut actions);
             code_action_import_module(module, &lines, &params, &this.error, &mut actions);
             code_action_add_missing_patterns(module, &lines, &params, &this.error, &mut actions);
+            actions
+                .extend(RemoveUnreachableCaseClauses::new(module, &lines, &params).code_actions());
+            actions.extend(CollapseNestedCase::new(module, &lines, &params).code_actions());
             code_action_inexhaustive_let_to_case(
                 module,
                 &lines,
@@ -425,7 +434,10 @@ where
             actions.extend(InterpolateString::new(module, &lines, &params).code_actions());
             actions.extend(ExtractVariable::new(module, &lines, &params).code_actions());
             actions.extend(ExtractConstant::new(module, &lines, &params).code_actions());
-            actions.extend(GenerateFunction::new(module, &lines, &params).code_actions());
+            actions.extend(
+                GenerateFunction::new(module, &this.compiler.modules, &lines, &params)
+                    .code_actions(),
+            );
             actions.extend(
                 GenerateVariant::new(module, &this.compiler, &lines, &params).code_actions(),
             );
@@ -434,9 +446,12 @@ where
             actions.extend(
                 PatternMatchOnValue::new(module, &lines, &params, &this.compiler).code_actions(),
             );
+            actions.extend(AddOmittedLabels::new(module, &lines, &params).code_actions());
             actions.extend(InlineVariable::new(module, &lines, &params).code_actions());
             actions.extend(WrapInBlock::new(module, &lines, &params).code_actions());
             actions.extend(RemoveBlock::new(module, &lines, &params).code_actions());
+            actions.extend(RemovePrivateOpaque::new(module, &lines, &params).code_actions());
+            actions.extend(ExtractFunction::new(module, &lines, &params).code_actions());
             GenerateDynamicDecoder::new(module, &lines, &params, &mut actions).code_actions();
             GenerateJsonEncoder::new(
                 module,
@@ -642,11 +657,16 @@ where
                 }) if location.contains(byte_index) => match origin.map(|origin| origin.syntax) {
                     Some(VariableSyntax::Generated) => None,
                     Some(
-                        VariableSyntax::Variable { .. }
-                        | VariableSyntax::AssignmentPattern
-                        | VariableSyntax::LabelShorthand(_),
-                    )
-                    | None => success_response(location),
+                        VariableSyntax::Variable(label) | VariableSyntax::LabelShorthand(label),
+                    ) => success_response(SrcSpan {
+                        start: location.start,
+                        end: label
+                            .len()
+                            .try_into()
+                            .map(|len: u32| location.start + len)
+                            .unwrap_or(location.end),
+                    }),
+                    Some(VariableSyntax::AssignmentPattern) | None => success_response(location),
                 },
                 Some(
                     Referenced::ModuleValue {
@@ -794,7 +814,8 @@ where
                     )
                     | None => {
                         let variable_references =
-                            find_variable_references(&module.ast, definition_location, name);
+                            FindVariableReferences::new(definition_location, name)
+                                .find_in_module(&module.ast);
 
                         let mut reference_locations =
                             Vec::with_capacity(variable_references.len() + 1);
