@@ -1,4 +1,4 @@
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 
 use ecow::EcoString;
 use lsp_types::Location;
@@ -342,46 +342,56 @@ fn find_references_in_module(
     }
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub struct VariableReference {
     pub location: SrcSpan,
     pub kind: VariableReferenceKind,
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum VariableReferenceKind {
     Variable,
     LabelShorthand,
 }
 
-pub fn find_variable_references(
-    module: &TypedModule,
-    definition_location: SrcSpan,
-    name: EcoString,
-) -> Vec<VariableReference> {
-    let mut finder = FindVariableReferences {
-        references: Vec::new(),
-        definition_location,
-        alternative_variable: AlternativeVariable::Ignore,
-        name,
-    };
-    finder.visit_typed_module(module);
-    finder.references
-}
-
 /// How to treat variables defined in alternative patterns
 enum AlternativeVariable {
     Track,
     Ignore,
 }
 
-struct FindVariableReferences {
-    references: Vec<VariableReference>,
+pub struct FindVariableReferences {
+    // Due to the structure of some AST nodes (for example, record updates),
+    // when we traverse the AST it is possible to accidentally duplicate references.
+    // To avoid this, we use a `HashSet` instead of a `Vec` here.
+    // See: https://github.com/gleam-lang/gleam/issues/4859 and the linked PR.
+    references: HashSet<VariableReference>,
     definition_location: SrcSpan,
     alternative_variable: AlternativeVariable,
     name: EcoString,
 }
 
+impl FindVariableReferences {
+    pub fn new(variable_definition_location: SrcSpan, variable_name: EcoString) -> Self {
+        Self {
+            references: HashSet::new(),
+            definition_location: variable_definition_location,
+            alternative_variable: AlternativeVariable::Ignore,
+            name: variable_name,
+        }
+    }
+
+    pub fn find_in_module(mut self, module: &TypedModule) -> HashSet<VariableReference> {
+        self.visit_typed_module(module);
+        self.references
+    }
+
+    pub fn find(mut self, expression: &TypedExpr) -> HashSet<VariableReference> {
+        self.visit_typed_expr(expression);
+        self.references
+    }
+}
+
 impl<'ast> Visit<'ast> for FindVariableReferences {
     fn visit_typed_function(&mut self, fun: &'ast ast::TypedFunction) {
         if fun.full_location().contains(self.definition_location.start) {
@@ -400,10 +410,10 @@ impl<'ast> Visit<'ast> for FindVariableReferences {
                 location: definition_location,
                 ..
             } if definition_location == self.definition_location => {
-                self.references.push(VariableReference {
+                _ = self.references.insert(VariableReference {
                     location: *location,
                     kind: VariableReferenceKind::Variable,
-                })
+                });
             }
             _ => {}
         }
@@ -417,10 +427,10 @@ impl<'ast> Visit<'ast> for FindVariableReferences {
         definition_location: &'ast SrcSpan,
     ) {
         if *definition_location == self.definition_location {
-            self.references.push(VariableReference {
+            _ = self.references.insert(VariableReference {
                 location: *location,
                 kind: VariableReferenceKind::Variable,
-            })
+            });
         }
     }
 
@@ -468,7 +478,7 @@ impl<'ast> Visit<'ast> for FindVariableReferences {
             AlternativeVariable::Track
                 if *name == self.name && *location != self.definition_location =>
             {
-                self.references.push(VariableReference {
+                _ = self.references.insert(VariableReference {
                     location: *location,
                     kind: VariableReferenceKind::Variable,
                 });
@@ -493,10 +503,10 @@ impl<'ast> Visit<'ast> for FindVariableReferences {
                 location: definition_location,
                 ..
             } if *definition_location == self.definition_location => {
-                self.references.push(VariableReference {
+                _ = self.references.insert(VariableReference {
                     location: *location,
                     kind: VariableReferenceKind::Variable,
-                })
+                });
             }
             _ => {}
         }
@@ -516,7 +526,7 @@ impl<'ast> Visit<'ast> for FindVariableReferences {
                 } if arg.uses_label_shorthand()
                     && *definition_location == self.definition_location =>
                 {
-                    self.references.push(VariableReference {
+                    _ = self.references.insert(VariableReference {
                         location: *location,
                         kind: VariableReferenceKind::LabelShorthand,
                     });
