@@ -325,6 +325,24 @@ pub fn main() {
     );
 }
 
+#[test]
+fn rename_local_variable_from_label_shorthand() {
+    assert_rename!(
+        "
+type Wibble {
+  Wibble(wibble: Int)
+}
+
+pub fn main() {
+  let wibble = todo
+  Wibble(wibble:)
+}
+",
+        "wobble",
+        find_position_of("wibble:)")
+    );
+}
+
 #[test]
 fn rename_local_variable_in_bit_array_pattern() {
     assert_rename!(
@@ -1508,3 +1526,75 @@ pub fn main() {
         find_position_of("Ok")
     );
 }
+
+#[test]
+fn rename_module_access_in_clause_guard() {
+    assert_rename!(
+        (
+            "wibble",
+            "
+import app
+
+pub fn main() {
+  case app.something {
+    thing if thing == app.something -> True
+    _ -> False
+  }
+}
+"
+        ),
+        "
+pub const something = 10
+",
+        "new_name",
+        find_position_of("something")
+    );
+}
+
+#[test]
+fn rename_variable_used_in_record_update() {
+    assert_rename!(
+        "
+type Wibble {
+  Wibble(a: Int, b: Int, c: Int)
+}
+
+fn wibble(wibble: Wibble) {
+  Wibble(..wibble, c: 1)
+}
+",
+        "value",
+        find_position_of("wibble:")
+    );
+}
+
+//https://github.com/gleam-lang/gleam/issues/4941
+#[test]
+fn rename_external_function() {
+    assert_rename!(
+        r#"
+pub fn main() { wibble() }
+
+@external(erlang, "a", "a")
+fn wibble() -> Nil
+"#,
+        "new_name",
+        find_position_of("wibble").nth_occurrence(2)
+    );
+}
+
+#[test]
+fn rename_external_javascript_function_with_pure_gleam_fallback() {
+    assert_rename!(
+        r#"
+pub fn main() { wibble() }
+
+@external(javascript, "a", "a")
+fn wibble() -> Nil {
+  Nil
+}
+"#,
+        "new_name",
+        find_position_of("wibble").nth_occurrence(2)
+    );
+}
