@@ -90,6 +90,7 @@ impl ModuleDecoder {
             documentation: self.string_list(reader.get_documentation()?)?,
             contains_echo: reader.get_contains_echo(),
             references: self.references(reader.get_references()?)?,
+            inline_functions: HashMap::new(),
         })
     }
 
@@ -196,6 +197,7 @@ impl ModuleDecoder {
                 message: self.string(deprecation)?,
             }
         };
+        let parameters = read_vec!(&reader.get_parameters()?, self, type_);
         Ok(TypeAliasConstructor {
             publicity: self.publicity(reader.get_publicity()?)?,
             origin: self.src_span(&reader.get_origin()?)?,
@@ -204,6 +206,7 @@ impl ModuleDecoder {
             deprecation,
             documentation: self.optional_string(self.str(reader.get_documentation()?)?),
             arity: reader.get_arity() as usize,
+            parameters,
         })
     }
 
@@ -223,9 +226,10 @@ impl ModuleDecoder {
         let name = self.string(reader.get_name()?)?;
         let arguments = read_vec!(&reader.get_parameters()?, self, type_);
         let inferred_variant = self.inferred_variant(&reader.get_inferred_variant()?)?;
+        let publicity = self.publicity(reader.get_publicity()?)?;
 
         Ok(Arc::new(Type::Named {
-            publicity: Publicity::Public,
+            publicity,
             package,
             module,
             name,
@@ -315,6 +319,7 @@ impl ModuleDecoder {
         Ok(TypeValueConstructorField {
             type_: self.type_(&reader.get_type()?)?,
             label: self.optional_string(self.str(reader.get_label()?)?),
+            documentation: self.optional_string(self.str(reader.get_documentation()?)?),
         })
     }
 
@@ -697,6 +702,7 @@ impl ModuleDecoder {
             index: reader.get_index() as u64,
             label: self.string(reader.get_label()?)?,
             type_: self.type_(&reader.get_type()?)?,
+            documentation: self.optional_string(self.str(reader.get_documentation()?)?),
         })
     }
 
