@@ -73,6 +73,7 @@ fn constant_module(constant: TypedConstant) -> ModuleInterface {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     }
 }
 
@@ -112,6 +113,7 @@ fn empty_module() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -140,6 +142,7 @@ fn with_line_numbers() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -176,6 +179,7 @@ fn module_with_private_type() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -212,6 +216,7 @@ fn module_with_app_type() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -248,6 +253,7 @@ fn module_with_fn_type() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -284,6 +290,7 @@ fn module_with_tuple_type() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -325,6 +332,7 @@ fn module_with_generic_type() {
             documentation: Vec::new(),
             contains_echo: false,
             references: References::default(),
+            inline_functions: HashMap::new(),
         }
     }
 
@@ -366,6 +374,7 @@ fn module_with_type_links() {
             documentation: Vec::new(),
             contains_echo: false,
             references: References::default(),
+            inline_functions: HashMap::new(),
         }
     }
 
@@ -407,6 +416,7 @@ fn module_with_type_constructor_documentation() {
             documentation: Vec::new(),
             contains_echo: false,
             references: References::default(),
+            inline_functions: HashMap::new(),
         }
     }
 
@@ -451,6 +461,7 @@ fn module_with_type_constructor_origin() {
             documentation: Vec::new(),
             contains_echo: false,
             references: References::default(),
+            inline_functions: HashMap::new(),
         }
     }
 
@@ -489,6 +500,7 @@ fn module_type_to_constructors_mapping() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -543,6 +555,7 @@ fn module_fn_value() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -598,6 +611,7 @@ fn deprecated_module_fn_value() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -651,6 +665,7 @@ fn private_module_fn_value() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -706,6 +721,7 @@ fn module_fn_value_regression() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -760,6 +776,7 @@ fn module_fn_value_with_field_map() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -808,6 +825,7 @@ fn record_value() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -863,6 +881,7 @@ fn record_value_with_field_map() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -877,6 +896,7 @@ fn accessors() {
                 index: 6,
                 label: "siiixxx".into(),
                 type_: type_::nil(),
+                documentation: Some("Here is some documentation".into()),
             },
         ),
         (
@@ -885,6 +905,7 @@ fn accessors() {
                 index: 5,
                 label: "fiveee".into(),
                 type_: type_::float(),
+                documentation: None,
             },
         ),
     ];
@@ -895,6 +916,7 @@ fn accessors() {
             index: 1,
             label: "ok".into(),
             type_: type_::float(),
+            documentation: Some("Documentation for the ok field".into()),
         },
     )];
 
@@ -936,6 +958,7 @@ fn accessors() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -950,6 +973,7 @@ fn private_accessors() {
                 index: 6,
                 label: "siiixxx".into(),
                 type_: type_::nil(),
+                documentation: None,
             },
         ),
         (
@@ -958,6 +982,7 @@ fn private_accessors() {
                 index: 5,
                 label: "fiveee".into(),
                 type_: type_::float(),
+                documentation: None,
             },
         ),
     ];
@@ -968,6 +993,7 @@ fn private_accessors() {
             index: 1,
             label: "ok".into(),
             type_: type_::float(),
+            documentation: None,
         },
     )];
 
@@ -1008,6 +1034,7 @@ fn private_accessors() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1239,6 +1266,7 @@ fn constant_var() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1436,6 +1464,7 @@ fn deprecated_type() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1488,6 +1517,7 @@ fn module_fn_value_with_external_implementations() {
         documentation: Vec::new(),
         contains_echo: false,
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1512,6 +1542,7 @@ fn module_containing_echo() {
         documentation: Vec::new(),
         contains_echo: true,
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1568,6 +1599,7 @@ fn internal_module_fn() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1627,6 +1659,7 @@ fn internal_annotated_module_fn() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1652,14 +1685,17 @@ fn type_variable_ids_in_constructors_are_shared() {
                         TypeValueConstructorField {
                             type_: type_::generic_var(6),
                             label: None,
+                            documentation: Some("Here's some documentation".into()),
                         },
                         TypeValueConstructorField {
                             type_: type_::int(),
                             label: None,
+                            documentation: None,
                         },
                         TypeValueConstructorField {
                             type_: type_::tuple(vec![type_::generic_var(4), type_::generic_var(5)]),
                             label: None,
+                            documentation: None,
                         },
                     ],
                     documentation: None,
@@ -1677,6 +1713,7 @@ fn type_variable_ids_in_constructors_are_shared() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     let expected = HashMap::from([(
@@ -1689,14 +1726,17 @@ fn type_variable_ids_in_constructors_are_shared() {
                     TypeValueConstructorField {
                         type_: type_::generic_var(0),
                         label: None,
+                        documentation: Some("Here's some documentation".into()),
                     },
                     TypeValueConstructorField {
                         type_: type_::int(),
                         label: None,
+                        documentation: None,
                     },
                     TypeValueConstructorField {
                         type_: type_::tuple(vec![type_::generic_var(1), type_::generic_var(2)]),
                         label: None,
+                        documentation: None,
                     },
                 ],
                 documentation: None,
@@ -1721,14 +1761,18 @@ fn type_with_inferred_variant() {
             "Wibble".into(),
             TypeConstructor {
                 type_: Arc::new(Type::Named {
-                    publicity: Publicity::Public,
+                    publicity: Publicity::Internal {
+                        attribute_location: None,
+                    },
                     package: "some_package".into(),
                     module: "the/module".into(),
                     name: "Wibble".into(),
                     arguments: Vec::new(),
                     inferred_variant: Some(1),
                 }),
-                publicity: Publicity::Public,
+                publicity: Publicity::Internal {
+                    attribute_location: Some(SrcSpan::new(0, 10)),
+                },
                 origin: Default::default(),
                 module: "the/module".into(),
                 parameters: vec![],
@@ -1746,6 +1790,7 @@ fn type_with_inferred_variant() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1775,6 +1820,7 @@ fn module_with_type_aliases() {
                 deprecation: Deprecation::NotDeprecated,
                 documentation: Some("Some documentation".into()),
                 origin: Default::default(),
+                parameters: vec![type_::generic_var(0)],
             },
         )]
         .into(),
@@ -1782,6 +1828,7 @@ fn module_with_type_aliases() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1810,6 +1857,7 @@ fn module_with_documentation() {
         contains_echo: false,
 
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1845,6 +1893,7 @@ fn module_with_opaque_type() {
         documentation: Vec::new(),
         contains_echo: false,
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1935,6 +1984,7 @@ fn module_with_references() {
             )]
             .into(),
         },
+        inline_functions: HashMap::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
