@@ -104,7 +104,6 @@ pub enum ParseErrorType {
     CallInClauseGuard, // case x { _ if f() -> 1 }
     IfExpression,
     ConstantRecordConstructorNoArguments, // const x = Record()
-    TypeConstructorNoArguments,           // let a : Int()
     TypeDefinitionNoArguments,            // pub type Wibble() { ... }
     UnknownAttributeRecordVariant, // an attribute was used that is not know for a custom type variant
     // a Python-like import was written, such as `import gleam.io`, instead of `import gleam/io`
@@ -112,6 +111,23 @@ pub enum ParseErrorType {
         module: EcoString,
         item: EcoString,
     },
+    /// This can happen when there's an empty block in a case clause guard.
+    /// For example: `_ if a == {}`
+    EmptyGuardBlock,
+    // When the use tries to define a constant inside a function
+    ConstantInsideFunction,
+    FunctionDefinitionAngleGenerics, // fn something<T>() { ... }
+    // let a: List<String> = []
+    TypeUsageAngleGenerics {
+        module: Option<EcoString>,
+        name: EcoString,
+        arguments: Vec<TypeAst>,
+    },
+    // type Something<T> {
+    TypeDefinitionAngleGenerics {
+        name: EcoString,
+        arguments: Vec<EcoString>,
+    },
 }
 
 pub(crate) struct ParseErrorDetails {
@@ -594,13 +610,6 @@ utf16_codepoint, utf32_codepoint, signed, unsigned, big, little, native, size, u
                 extra_labels: vec![],
             },
 
-            ParseErrorType::TypeConstructorNoArguments => ParseErrorDetails {
-                text: "A type constructor must be passed arguments.".into(),
-                hint: None,
-                label_text: "I was expecting arguments here".into(),
-                extra_labels: vec![],
-            },
-
             ParseErrorType::TypeDefinitionNoArguments => ParseErrorDetails {
                 text: "A generic type must have at least a generic parameter.".into(),
                 hint: Some("If a type is not generic you should omit the `()`.".into()),
@@ -627,6 +636,91 @@ utf16_codepoint, utf32_codepoint, signed, unsigned, big, little, native, size, u
                 label_text: "I was expecting either `/` or `.{` here.".into(),
                 extra_labels: vec![],
             },
+
+            ParseErrorType::EmptyGuardBlock => ParseErrorDetails {
+                text: "".into(),
+                hint: None,
+                label_text: "A clause guard block cannot be empty".into(),
+                extra_labels: vec![],
+            },
+
+            ParseErrorType::ConstantInsideFunction => ParseErrorDetails {
+                text: wrap(
+                    "All variables are immutable in Gleam, so constants inside \
+functions are not necessary.",
+                ),
+                hint: Some(
+                    "Either move this into the global scope or use `let` binding instead.".into(),
+                ),
+                label_text: "Constants are not allowed inside functions".into(),
+                extra_labels: vec![],
+            },
+
+            ParseErrorType::FunctionDefinitionAngleGenerics => ParseErrorDetails {
+                text: "\
+Generic function type variables do not need to be predeclared like they
+would be in some other languages, instead they are written with lowercase
+names.
+
+    fn example(argument: generic) -> generic
+
+See: https://tour.gleam.run/functions/generic-functions/"
+                    .into(),
+                hint: None,
+                label_text: "I was expecting `(` here.".into(),
+                extra_labels: vec![],
+            },
+
+            ParseErrorType::TypeUsageAngleGenerics {
+                module,
+                name,
+                arguments,
+            } => {
+                let type_arguments = arguments
+                    .iter()
+                    .map(|argument| {
+                        let mut argument_string = EcoString::new();
+                        argument.print(&mut argument_string);
+                        argument_string
+                    })
+                    .join(", ");
+                let replacement_type = match module {
+                    Some(module) => format!("{module}.{name}({type_arguments})"),
+                    None => format!("{name}({type_arguments})"),
+                };
+
+                ParseErrorDetails {
+                    text: format!(
+                        "\
+Type parameters use lowercase names and are surrounded by parentheses.
+
+    {replacement_type}
+
+See: https://tour.gleam.run/data-types/generic-custom-types/"
+                    ),
+                    hint: None,
+                    label_text: "I was expecting `(` here.".into(),
+                    extra_labels: vec![],
+                }
+            }
+
+            ParseErrorType::TypeDefinitionAngleGenerics { name, arguments } => {
+                let comma_separated_arguments = arguments.join(", ");
+
+                ParseErrorDetails {
+                    text: format!(
+                        "\
+Type parameters use lowercase names and are surrounded by parentheses.
+
+    type {name}({comma_separated_arguments}) {{
+
+See: https://tour.gleam.run/data-types/generic-custom-types/"
+                    ),
+                    hint: None,
+                    label_text: "I was expecting `(` here.".into(),
+                    extra_labels: vec![],
+                }
+            }
         }
     }
 }
