@@ -1740,20 +1740,6 @@ pub type A() {
     );
 }
 
-#[test]
-fn missing_type_constructor_arguments_in_type_annotation_1() {
-    assert_module_error!("pub fn main() -> Int() {}");
-}
-
-#[test]
-fn missing_type_constructor_arguments_in_type_annotation_2() {
-    assert_module_error!(
-        "pub fn main() {
-  let a: Int() = todo
-}"
-    );
-}
-
 #[test]
 fn tuple_without_hash() {
     assert_module_error!(
@@ -1907,3 +1893,102 @@ fn operator_in_pattern_size() {
 fn correct_precedence_in_pattern_size() {
     assert_parse!("let assert <<size, payload:size(size + 2 * 8)>> = <<>>");
 }
+
+#[test]
+fn private_opaque_type_is_parsed() {
+    assert_parse_module!("opaque type Wibble { Wobble }");
+}
+
+#[test]
+fn case_guard_with_nested_blocks() {
+    assert_parse!(
+        "case 1 {
+  _ if { 1 || { 1 || 1 } } || 1  -> 1
+}"
+    );
+}
+
+#[test]
+fn case_guard_with_empty_block() {
+    assert_error!(
+        "case 1 {
+  _ if a || {}  -> 1
+}"
+    );
+}
+
+#[test]
+fn constant_inside_function() {
+    assert_module_error!(
+        "
+pub fn main() {
+  const x = 10
+  x
+}
+"
+    );
+}
+
+#[test]
+fn function_definition_angle_generics_error() {
+    assert_module_error!("fn id<T>(x: T) { x }");
+}
+
+#[test]
+fn type_angle_generics_usage_without_module_error() {
+    assert_error!("let list: List<Int, String> = []");
+}
+
+#[test]
+fn type_angle_generics_usage_with_module_error() {
+    assert_error!("let set: set.Set<Int> = set.new()");
+}
+
+#[test]
+fn type_angle_generics_definition_error() {
+    assert_module_error!(
+        r#"
+type Either<a, b> {
+    This(a)
+    That(b)
+}
+"#
+    );
+}
+
+#[test]
+fn type_angle_generics_definition_with_upname_error() {
+    assert_module_error!(
+        r#"
+type Either<A, B> {
+    This(A)
+    That(B)
+}
+"#
+    );
+}
+
+#[test]
+fn type_angle_generics_definition_error_fallback() {
+    // Example of a more incorrect syntax, where Gleam doesn't make a suggestion.
+    // In this case, an example type argument is used instead.
+    assert_module_error!(
+        r#"
+type Either<type A, type B> {
+    This(A)
+    That(B)
+}
+"#
+    );
+}
+
+#[test]
+fn wrong_type_of_comments_with_hash() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  # a python-style comment
+}
+"#
+    );
+}
