@@ -31,6 +31,7 @@ use crate::{
     },
     bit_array,
     build::{Origin, Target},
+    inline::InlinableFunction,
     line_numbers::LineNumbers,
     reference::ReferenceMap,
     type_::expression::Implementations,
@@ -319,6 +320,14 @@ impl Type {
         }
     }
 
+    pub fn is_list(&self) -> bool {
+        match self {
+            Self::Named { module, name, .. } if "List" == name && is_prelude_module(module) => true,
+            Self::Var { type_ } => type_.borrow().is_list(),
+            _ => false,
+        }
+    }
+
     pub fn named_type_name(&self) -> Option<(EcoString, EcoString)> {
         match self {
             Self::Named { module, name, .. } => Some((module.clone(), name.clone())),
@@ -630,10 +639,10 @@ impl TypeVar {
 }
 
 pub fn collapse_links(t: Arc<Type>) -> Arc<Type> {
-    if let Type::Var { type_ } = t.deref() {
-        if let TypeVar::Link { type_ } = type_.borrow().deref() {
-            return collapse_links(type_.clone());
-        }
+    if let Type::Var { type_ } = t.deref()
+        && let TypeVar::Link { type_ } = type_.borrow().deref()
+    {
+        return collapse_links(type_.clone());
     }
     t
 }
@@ -663,6 +672,7 @@ pub struct RecordAccessor {
     pub index: u64,
     pub label: EcoString,
     pub type_: Arc<Type>,
+    pub documentation: Option<EcoString>,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -994,6 +1004,8 @@ pub struct ModuleInterface {
     /// Wether there's any echo in the module.
     pub contains_echo: bool,
     pub references: References,
+    /// Functions which can be inlined
+    pub inline_functions: HashMap<EcoString, InlinableFunction>,
 }
 
 impl ModuleInterface {
@@ -1080,6 +1092,7 @@ pub struct TypeValueConstructorField {
     /// This type of this parameter
     pub type_: Arc<Type>,
     pub label: Option<EcoString>,
+    pub documentation: Option<EcoString>,
 }
 
 impl ModuleInterface {
@@ -1236,6 +1249,13 @@ impl TypeVar {
         }
     }
 
+    pub fn is_list(&self) -> bool {
+        match self {
+            TypeVar::Link { type_ } => type_.is_list(),
+            TypeVar::Unbound { .. } | TypeVar::Generic { .. } => false,
+        }
+    }
+
     pub fn result_ok_type(&self) -> Option<Arc<Type>> {
         match self {
             TypeVar::Link { type_ } => type_.result_ok_type(),
@@ -1475,6 +1495,7 @@ pub struct TypeAliasConstructor {
     pub deprecation: Deprecation,
     pub documentation: Option<EcoString>,
     pub origin: SrcSpan,
+    pub parameters: Vec<Arc<Type>>,
 }
 
 impl ValueConstructor {
