@@ -22,6 +22,7 @@ pub struct EnvironmentArguments<'a> {
     pub importable_modules: &'a im::HashMap<EcoString, ModuleInterface>,
     pub target_support: TargetSupport,
     pub current_origin: Origin,
+    pub dev_dependencies: &'a HashSet<EcoString>,
 }
 
 impl<'a> EnvironmentArguments<'a> {
@@ -77,8 +78,7 @@ pub struct Environment<'a> {
     /// local_variable_usages is a stack of scopes. When a local variable is created it is
     /// added to the top scope. When a local variable is used we crawl down the scope
     /// stack for a variable with that name and mark it as used.
-    /// NOTE: The bool in the tuple here tracks if the variable has been used
-    pub local_variable_usages: Vec<HashMap<EcoString, (VariableOrigin, SrcSpan, bool)>>,
+    pub local_variable_usages: Vec<HashMap<EcoString, VariableUsage>>,
 
     /// Used to determine if all functions/constants need to support the current
     /// compilation target.
@@ -90,6 +90,16 @@ pub struct Environment<'a> {
     pub echo_found: bool,
 
     pub references: ReferenceTracker,
+
+    pub dev_dependencies: &'a HashSet<EcoString>,
+}
+
+#[derive(Debug)]
+pub struct VariableUsage {
+    origin: VariableOrigin,
+    location: SrcSpan,
+    usages: usize,
+    recursive_usages: usize,
 }
 
 impl<'a> Environment<'a> {
@@ -103,27 +113,17 @@ impl<'a> Environment<'a> {
             importable_modules,
             target_support,
             current_origin: origin,
+            dev_dependencies,
         }: EnvironmentArguments<'a>,
     ) -> Self {
         let prelude = importable_modules
             .get(PRELUDE_MODULE_NAME)
             .expect("Unable to find prelude in importable modules");
 
-        let mut names = Names::new();
-        for name in prelude.values.keys() {
-            names.named_constructor_in_scope(
-                PRELUDE_MODULE_NAME.into(),
-                name.clone(),
-                name.clone(),
-            );
-        }
-
-        for name in prelude.types.keys() {
-            names.named_type_in_scope(PRELUDE_MODULE_NAME.into(), name.clone(), name.clone());
-        }
+        let names = Self::build_names(prelude, importable_modules);
 
         Self {
-            current_package: current_package.clone(),
+            current_package,
             gleam_version,
             previous_id: ids.next(),
             ids,
@@ -146,8 +146,45 @@ impl<'a> Environment<'a> {
             module_type_aliases: HashMap::new(),
             echo_found: false,
             references: ReferenceTracker::new(),
+            dev_dependencies,
         }
     }
+
+    fn build_names(
+        prelude: &ModuleInterface,
+        importable_modules: &im::HashMap<EcoString, ModuleInterface>,
+    ) -> Names {
+        let mut names = Names::new();
+
+        // Insert prelude types and values into scope
+        for name in prelude.values.keys() {
+            names.named_constructor_in_scope(
+                PRELUDE_MODULE_NAME.into(),
+                name.clone(),
+                name.clone(),
+            );
+        }
+        for name in prelude.types.keys() {
+            names.named_type_in_scope(PRELUDE_MODULE_NAME.into(), name.clone(), name.clone());
+        }
+
+        // Find potential type aliases which reexport internal types
+        for module in importable_modules.values() {
+            // Internal modules are not part of the public API so they are also
+            // not considered.
+            if module.is_internal {
+                continue;
+            }
+            for (alias_name, alias) in module.type_aliases.iter() {
+                // An alias can only be a public reexport if it is public.
+                if alias.publicity.is_public() {
+                    names.maybe_register_reexport_alias(&module.package, alias_name, alias);
+                }
+            }
+        }
+
+        names
+    }
 }
 
 #[derive(Debug)]
@@ -628,30 +665,63 @@ impl Environment<'_> {
         location: SrcSpan,
         problems: &mut Problems,
     ) {
-        if let Some((origin, location, false)) = self
+        if let Some(VariableUsage {
+            origin,
+            location,
+            usages: 0,
+            recursive_usages,
+        }) = self
             .local_variable_usages
             .last_mut()
             .expect("Attempted to access non-existent entity usages scope")
-            .insert(name.clone(), (origin, location, false))
+            .insert(
+                name.clone(),
+                VariableUsage {
+                    origin,
+                    location,
+                    usages: 0,
+                    recursive_usages: 0,
+                },
+            )
         {
             // an entity was overwritten in the top most scope without being used
             let mut unused = HashMap::with_capacity(1);
-            let _ = unused.insert(name, (origin, location, false));
+            let _ = unused.insert(
+                name,
+                VariableUsage {
+                    origin,
+                    location,
+                    usages: 0,
+                    recursive_usages,
+                },
+            );
             self.handle_unused_variables(unused, problems);
         }
     }
 
     /// Increments an entity's usage in the current or nearest enclosing scope
     pub fn increment_usage(&mut self, name: &EcoString) {
-        let name = name.clone();
-
-        if let Some((_, _, used)) = self
+        if let Some(VariableUsage { usages, .. }) = self
             .local_variable_usages
             .iter_mut()
             .rev()
-            .find_map(|scope| scope.get_mut(&name))
+            .find_map(|scope| scope.get_mut(name))
         {
-            *used = true;
+            *usages += 1;
+        }
+    }
+
+    /// Marks an argument as being passed recursively to a function call.
+    pub fn increment_recursive_usage(&mut self, name: &EcoString) {
+        if let Some(VariableUsage {
+            recursive_usages, ..
+        }) = self
+            .local_variable_usages
+            .iter_mut()
+            .rev()
+            .find_map(|scope| scope.get_mut(name))
+        {
+            *recursive_usages += 1;
         }
     }
 
@@ -765,13 +835,25 @@ impl Environment<'_> {
 
     fn handle_unused_variables(
         &mut self,
-        unused: HashMap<EcoString, (VariableOrigin, SrcSpan, bool)>,
+        unused: HashMap<EcoString, VariableUsage>,
         problems: &mut Problems,
     ) {
-        for (origin, location, used) in unused.into_values() {
-            if !used {
+        for VariableUsage {
+            origin,
+            location,
+            usages,
+            recursive_usages,
+        } in unused.into_values()
+        {
+            if usages == 0 {
                 problems.warning(Warning::UnusedVariable { location, origin });
             }
+            // If the function parameter is actually used somewhere, but all the
+            // usages are just passing it along in a recursive call, then it
+            // counts as being unused too.
+            else if origin.is_function_parameter() && recursive_usages == usages {
+                problems.warning(Warning::UnusedRecursiveArgument { location });
+            }
         }
     }
 
@@ -877,10 +959,10 @@ pub fn unify(t1: Arc<Type>, t2: Arc<Type>) -> Result<(), UnifyError> {
     }
 
     // Collapse right hand side type links. Left hand side will be collapsed in the next block.
-    if let Type::Var { type_ } = t2.deref() {
-        if let TypeVar::Link { type_ } = type_.borrow().deref() {
-            return unify(t1, type_.clone());
-        }
+    if let Type::Var { type_ } = t2.deref()
+        && let TypeVar::Link { type_ } = type_.borrow().deref()
+    {
+        return unify(t1, type_.clone());
     }
 
     if let Type::Var { type_ } = t1.deref() {
@@ -899,11 +981,11 @@ pub fn unify(t1: Arc<Type>, t2: Arc<Type>) -> Result<(), UnifyError> {
             }
 
             TypeVar::Generic { id } => {
-                if let Type::Var { type_ } = t2.deref() {
-                    if type_.borrow().is_unbound() {
-                        *type_.borrow_mut() = TypeVar::Generic { id: *id };
-                        return Ok(());
-                    }
+                if let Type::Var { type_ } = t2.deref()
+                    && type_.borrow().is_unbound()
+                {
+                    *type_.borrow_mut() = TypeVar::Generic { id: *id };
+                    return Ok(());
                 }
                 Action::CouldNotUnify
             }
