@@ -5,6 +5,7 @@ use super::{
 use crate::{
     ast::{BinOp, BitArraySegmentTruncation, Layer, SrcSpan, TodoKind},
     build::Target,
+    exhaustiveness::ImpossibleBitArraySegmentPattern,
     type_::{Type, expression::ComparisonOutcome},
 };
 
@@ -61,12 +62,6 @@ impl Problems {
     }
 }
 
-#[derive(Debug, Eq, PartialEq, Clone)]
-pub struct UnknownType {
-    pub location: SrcSpan,
-    pub name: EcoString,
-}
-
 /// This is used by the unknown record field error to tell if an unknown field
 /// is a field appearing in another variant of the same type to provide a better
 /// error message explaining why it can't be accessed:
@@ -641,6 +636,34 @@ pub enum Error {
     NonUtf8StringAssignmentInBitArray {
         location: SrcSpan,
     },
+
+    /// This happens when a private type is marked as opaque. Only public types
+    /// can be opaque.
+    ///
+    /// ```gleam
+    /// opaque type Wibble {
+    ///   Wobble
+    /// }
+    /// ```
+    ///
+    PrivateOpaqueType {
+        location: SrcSpan,
+    },
+
+    SrcImportingDevDependency {
+        importing_module: EcoString,
+        imported_module: EcoString,
+        package: EcoString,
+        location: SrcSpan,
+    },
+
+    /// This happens when a type has no type parameters (for example `Int`) but
+    /// it is being used as a constructor: `Int()`, `Bool(a, b)`.
+    ///
+    TypeUsedAsAConstructor {
+        location: SrcSpan,
+        name: EcoString,
+    },
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -738,13 +761,21 @@ pub enum VariableSyntax {
     Generated,
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+#[derive(Debug, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
 /// The source of a variable, such as a `let` assignment, or function parameter.
 pub enum VariableDeclaration {
     LetPattern,
     UsePattern,
     ClausePattern,
-    FunctionParameter,
+    FunctionParameter {
+        /// The name of the function defining the parameter. This will be None
+        /// for parameters introduced by anoynmous functions: `fn(a) { a }`
+        ///
+        function_name: Option<EcoString>,
+        /// The index of the parameter in the function's parameter list.
+        ///
+        index: usize,
+    },
     Generated,
 }
 
@@ -768,6 +799,17 @@ impl VariableOrigin {
             declaration: VariableDeclaration::Generated,
         }
     }
+
+    pub fn is_function_parameter(&self) -> bool {
+        match self.declaration {
+            VariableDeclaration::FunctionParameter { .. } => true,
+
+            VariableDeclaration::LetPattern
+            | VariableDeclaration::UsePattern
+            | VariableDeclaration::ClausePattern
+            | VariableDeclaration::Generated => false,
+        }
+    }
 }
 
 #[derive(Debug, Eq, PartialEq, Clone, serde::Serialize, serde::Deserialize)]
@@ -870,6 +912,11 @@ pub enum Warning {
         reason: UnreachablePatternReason,
     },
 
+    UnusedDiscardPattern {
+        location: SrcSpan,
+        name: EcoString,
+    },
+
     /// This happens when someone tries to write a case expression where one of
     /// the subjects is a literal tuple, list or bit array for example:
     ///
@@ -942,8 +989,9 @@ pub enum Warning {
         location: SrcSpan,
     },
 
-    AssertAssignmentOnInferredVariant {
+    AssertAssignmentOnImpossiblePattern {
         location: SrcSpan,
+        reason: AssertImpossiblePattern,
     },
 
     /// When a `todo` or `panic` is used as a function instead of providing the
@@ -1046,6 +1094,46 @@ pub enum Warning {
         location: SrcSpan,
         outcome: ComparisonOutcome,
     },
+    /// When a function's argument is only ever used unchanged in recursive
+    /// calls. For example:
+    ///
+    /// ```gleam
+    /// pub fn wibble(x, n) {
+    /// //            ^ This argument is not needed,
+    ///   case n {
+    ///     0 -> Nil
+    ///     _ -> wibble(x, n - 1)
+    /// //              ^ It's only used in recursive calls!
+    ///   }
+    /// }
+    /// ```
+    ///
+    UnusedRecursiveArgument {
+        location: SrcSpan,
+    },
+}
+
+#[derive(Debug, Eq, PartialEq, Clone, serde::Serialize, serde::Deserialize)]
+pub enum AssertImpossiblePattern {
+    /// When `let assert`-ing on a variant that's different from the inferred
+    /// one.
+    ///
+    /// ```gleam
+    /// let assert Error(_) = Ok(_)
+    /// ```
+    ///
+    InferredVariant,
+
+    /// When `let assert`-ing on a pattern that will never match because it's
+    /// matching on impossible segment(s).
+    ///
+    /// ```gleam
+    /// let assert <<-2:unsigned>> = bit_array
+    /// ```
+    ///
+    ImpossibleSegments {
+        segments: Vec<ImpossibleBitArraySegmentPattern>,
+    },
 }
 
 #[derive(Debug, Eq, Copy, PartialEq, Clone, serde::Serialize, serde::Deserialize)]
@@ -1120,7 +1208,7 @@ pub enum TodoOrPanic {
     Panic,
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+#[derive(Debug, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
 pub enum UnreachablePatternReason {
     /// The clause is unreachable because a previous pattern
     /// matches the same case.
@@ -1129,6 +1217,9 @@ pub enum UnreachablePatternReason {
     /// of the custom type that we are matching on, and this matches
     /// against one of the variants we know it isn't.
     ImpossibleVariant,
+    /// The clause is unreachable because it is matching on a pattern segment
+    /// that we could tell is never going to match
+    ImpossibleSegments(Vec<ImpossibleBitArraySegmentPattern>),
 }
 
 impl Error {
@@ -1149,6 +1240,7 @@ impl Error {
             | Error::UnsafeRecordUpdate { location, .. }
             | Error::UnnecessarySpreadOperator { location, .. }
             | Error::IncorrectTypeArity { location, .. }
+            | Error::TypeUsedAsAConstructor { location, .. }
             | Error::CouldNotUnify { location, .. }
             | Error::RecursiveType { location, .. }
             | Error::DuplicateName {
@@ -1204,8 +1296,9 @@ impl Error {
             | Error::IntOperatorOnFloats { location, .. }
             | Error::StringConcatenationWithAddInt { location }
             | Error::DoubleVariableAssignmentInBitArray { location }
-            | Error::NonUtf8StringAssignmentInBitArray { location } => location.start,
-
+            | Error::NonUtf8StringAssignmentInBitArray { location }
+            | Error::PrivateOpaqueType { location }
+            | Error::SrcImportingDevDependency { location, .. } => location.start,
             Error::UnknownLabels { unknown, .. } => {
                 unknown.iter().map(|(_, s)| s.start).min().unwrap_or(0)
             }
@@ -1263,7 +1356,7 @@ impl Warning {
             | Warning::OpaqueExternalType { location, .. }
             | Warning::InternalTypeLeak { location, .. }
             | Warning::RedundantAssertAssignment { location, .. }
-            | Warning::AssertAssignmentOnInferredVariant { location, .. }
+            | Warning::AssertAssignmentOnImpossiblePattern { location, .. }
             | Warning::TodoOrPanicUsedAsFunction { location, .. }
             | Warning::UnreachableCodeAfterPanic { location, .. }
             | Warning::RedundantPipeFunctionCapture { location, .. }
@@ -1272,10 +1365,12 @@ impl Warning {
             | Warning::AssertLiteralBool { location, .. }
             | Warning::BitArraySegmentTruncatedValue { location, .. }
             | Warning::TopLevelDefinitionShadowsImport { location, .. }
+            | Warning::UnusedDiscardPattern { location, .. }
             | Warning::ModuleImportedTwice {
                 second: location, ..
             }
-            | Warning::RedundantComparison { location, .. } => *location,
+            | Warning::RedundantComparison { location, .. }
+            | Warning::UnusedRecursiveArgument { location, .. } => *location,
         }
     }
 
