@@ -315,20 +315,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
         // Unify each pattern in the multi-pattern with the corresponding subject
         let mut typed_multi = Vec::with_capacity(multi_pattern.len());
         for (pattern, subject) in multi_pattern.into_iter().zip(subjects) {
-            let subject_variable = match subject {
-                TypedExpr::Var {
-                    constructor:
-                        ValueConstructor {
-                            // Records should not be considered local variables
-                            // See: https://github.com/gleam-lang/gleam/issues/3861
-                            variant: ValueConstructorVariant::Record { .. },
-                            ..
-                        },
-                    ..
-                } => None,
-                TypedExpr::Var { name, .. } => Some(name.clone()),
-                _ => None,
-            };
+            let subject_variable = Self::subject_variable(subject);
 
             let pattern = self.unify(pattern, subject.type_(), subject_variable);
             typed_multi.push(pattern);
@@ -344,7 +331,15 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
         pattern: UntypedPattern,
         subject: &TypedExpr,
     ) -> TypedPattern {
-        let subject_variable = match subject {
+        let subject_variable = Self::subject_variable(subject);
+
+        let typed_pattern = self.unify(pattern, subject.type_(), subject_variable);
+        self.register_variables();
+        typed_pattern
+    }
+
+    fn subject_variable(subject: &TypedExpr) -> Option<EcoString> {
+        match subject {
             TypedExpr::Var {
                 constructor:
                     ValueConstructor {
@@ -356,12 +351,15 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 ..
             } => None,
             TypedExpr::Var { name, .. } => Some(name.clone()),
+            // If the subject of a `case` expression is something like
+            // `echo some_variable`, we still want to narrow the variant for
+            // `some_variable`.
+            TypedExpr::Echo {
+                expression: Some(subject),
+                ..
+            } => Self::subject_variable(subject),
             _ => None,
-        };
-
-        let typed_pattern = self.unify(pattern, subject.type_(), subject_variable);
-        self.register_variables();
-        typed_pattern
+        }
     }
 
     /// Register the variables bound in this pattern in the environment
@@ -694,6 +692,13 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 location,
             } => {
                 let pattern = self.unify(*pattern, type_, subject_variable);
+
+                if pattern.is_discard() {
+                    self.problems.warning(Warning::UnusedDiscardPattern {
+                        location,
+                        name: name.clone(),
+                    });
+                }
                 self.insert_variable(
                     &name,
                     pattern.type_().clone(),
@@ -881,7 +886,11 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             location,
                             name_location,
                             name,
-                            arguments: self.infer_pattern_call_arguments(pattern_arguments, &[]),
+                            arguments: self.infer_pattern_call_arguments(
+                                pattern_arguments,
+                                &[],
+                                None,
+                            ),
                             module,
                             constructor: Inferred::Unknown,
                             spread,
@@ -1001,29 +1010,29 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             }
                         }
 
-                        if let Some(spread_location) = spread {
-                            if let ValueConstructorVariant::Record { arity, .. } =
+                        if let Some(spread_location) = spread
+                            && let ValueConstructorVariant::Record { arity, .. } =
                                 &constructor.variant
-                            {
-                                while pattern_arguments.len() < usize::from(*arity) {
-                                    pattern_arguments.push(CallArg {
-                                        value: Pattern::Discard {
-                                            name: "_".into(),
-                                            location: spread_location,
-                                            type_: (),
-                                        },
+                        {
+                            while pattern_arguments.len() < usize::from(*arity) {
+                                pattern_arguments.push(CallArg {
+                                    value: Pattern::Discard {
+                                        name: "_".into(),
                                         location: spread_location,
-                                        label: None,
-                                        implicit: Some(ImplicitCallArgOrigin::PatternFieldSpread),
-                                    });
-                                }
-                            };
-                        }
+                                        type_: (),
+                                    },
+                                    location: spread_location,
+                                    label: None,
+                                    implicit: Some(ImplicitCallArgOrigin::PatternFieldSpread),
+                                });
+                            }
+                        };
                     }
                 }
 
                 let constructor_type = constructor.type_.clone();
                 let constructor_deprecation = constructor.deprecation.clone();
+                let constructor_field_map = constructor.field_map().cloned();
                 let pattern_constructor = match &constructor.variant {
                     ValueConstructorVariant::Record {
                         name,
@@ -1109,8 +1118,11 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             });
                         }
 
-                        let pattern_arguments =
-                            self.infer_pattern_call_arguments(pattern_arguments, arguments);
+                        let pattern_arguments = self.infer_pattern_call_arguments(
+                            pattern_arguments,
+                            arguments,
+                            constructor_field_map,
+                        );
 
                         Pattern::Constructor {
                             location,
@@ -1166,13 +1178,22 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
         &mut self,
         pattern_arguments: Vec<CallArg<UntypedPattern>>,
         expected_types: &[Arc<Type>],
+        field_map: Option<FieldMap>,
     ) -> Vec<CallArg<TypedPattern>> {
         pattern_arguments
             .into_iter()
             .enumerate()
             .map(|(index, arg)| {
+                let mut index = index;
                 if !arg.is_implicit() && arg.uses_label_shorthand() {
                     self.track_feature_usage(FeatureKind::LabelShorthandSyntax, arg.location);
+
+                    if let Some(field_map) = &field_map
+                        && let Some(label) = &arg.label
+                        && let Some(field) = field_map.fields.get(label)
+                    {
+                        index = *field as usize
+                    }
                 }
 
                 let CallArg {
@@ -1316,21 +1337,21 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
 
         // Then if the required version is not in the specified version for the
         // range we emit a warning highlighting the usage of the feature.
-        if let Some(gleam_version) = &self.environment.gleam_version {
-            if let Some(lowest_allowed_version) = gleam_version.lowest_version() {
-                // There is a version in the specified range that is lower than
-                // the one required by this feature! This means that the
-                // specified range is wrong and would allow someone to run a
-                // compiler that is too old to know of this feature.
-                if minimum_required_version > lowest_allowed_version {
-                    self.problems
-                        .warning(Warning::FeatureRequiresHigherGleamVersion {
-                            location,
-                            feature_kind,
-                            minimum_required_version: minimum_required_version.clone(),
-                            wrongfully_allowed_version: lowest_allowed_version,
-                        })
-                }
+        if let Some(gleam_version) = &self.environment.gleam_version
+            && let Some(lowest_allowed_version) = gleam_version.lowest_version()
+        {
+            // There is a version in the specified range that is lower than
+            // the one required by this feature! This means that the
+            // specified range is wrong and would allow someone to run a
+            // compiler that is too old to know of this feature.
+            if minimum_required_version > lowest_allowed_version {
+                self.problems
+                    .warning(Warning::FeatureRequiresHigherGleamVersion {
+                        location,
+                        feature_kind,
+                        minimum_required_version: minimum_required_version.clone(),
+                        wrongfully_allowed_version: lowest_allowed_version,
+                    })
             }
         }
 
