@@ -246,6 +246,7 @@ pub fn build_prelude(ids: &UniqueIdGenerator) -> ModuleInterface {
         documentation: Vec::new(),
         contains_echo: false,
         references: References::default(),
+        inline_functions: HashMap::new(),
     };
 
     for t in PreludeType::iter() {
@@ -445,6 +446,7 @@ pub fn build_prelude(ids: &UniqueIdGenerator) -> ModuleInterface {
                                 parameters: vec![TypeValueConstructorField {
                                     type_: result_value,
                                     label: None,
+                                    documentation: None,
                                 }],
                                 documentation: None,
                             },
@@ -453,6 +455,7 @@ pub fn build_prelude(ids: &UniqueIdGenerator) -> ModuleInterface {
                                 parameters: vec![TypeValueConstructorField {
                                     type_: result_error,
                                     label: None,
+                                    documentation: None,
                                 }],
                                 documentation: None,
                             },
