@@ -101,6 +101,22 @@ macro_rules! assert_module_error {
         output.push_str(&format!("-- main.gleam\n{}\n\n----- ERROR\n{error}", $src));
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
+
+    ($(($package:literal, $name:expr, $module_src:literal)),+, $src:literal $(,)?) => {
+        let error = $crate::type_::tests::module_error(
+            $src,
+            vec![
+                $(($package, $name, $module_src)),*
+            ],
+        );
+
+        let mut output = String::from("----- SOURCE CODE\n");
+        for (name, src) in [$(($name, $module_src)),*] {
+            output.push_str(&format!("-- {name}.gleam\n{src}\n\n"));
+        }
+        output.push_str(&format!("-- main.gleam\n{}\n\n----- ERROR\n{error}", $src));
+        insta::assert_snapshot!(insta::internals::AutoName, output, $src);
+    };
 }
 
 #[macro_export]
@@ -175,8 +191,7 @@ fn get_warnings(
         target,
         TargetSupport::NotEnforced,
         gleam_version,
-    )
-    .expect("Compilation should succeed");
+    );
     warnings.take().into_iter().collect_vec()
 }
 
@@ -352,6 +367,7 @@ fn compile_statement_sequence(
     // place.
     let _ = modules.insert(PRELUDE_MODULE_NAME.into(), build_prelude(&ids));
     let mut problems = Problems::new();
+    let dev_dependencies = HashSet::new();
     let mut environment = EnvironmentArguments {
         ids,
         current_package: "thepackage".into(),
@@ -361,6 +377,7 @@ fn compile_statement_sequence(
         importable_modules: &modules,
         target_support: TargetSupport::Enforced,
         current_origin: Origin::Src,
+        dev_dependencies: &dev_dependencies,
     }
     .build();
     let res = ExprTyper::new(
@@ -437,7 +454,7 @@ pub fn compile_module(
     src: &str,
     warnings: Option<Rc<dyn WarningEmitterIO>>,
     dep: Vec<DependencyModule<'_>>,
-) -> Result<TypedModule, (Vec<crate::type_::Error>, Names)> {
+) -> Outcome<TypedModule, Vec1<super::Error>> {
     compile_module_with_opts(
         module_name,
         src,
@@ -457,7 +474,7 @@ pub fn compile_module_with_opts(
     target: Target,
     target_support: TargetSupport,
     gleam_version: Option<Range<Version>>,
-) -> Result<TypedModule, (Vec<crate::type_::Error>, Names)> {
+) -> Outcome<TypedModule, Vec1<super::Error>> {
     let ids = UniqueIdGenerator::new();
     let mut modules = im::HashMap::new();
 
@@ -487,6 +504,7 @@ pub fn compile_module_with_opts(
             importable_modules: &modules,
             warnings: &TypeWarningEmitter::null(),
             direct_dependencies: &HashMap::new(),
+            dev_dependencies: &HashSet::new(),
             target_support,
             package_config: &config,
         }
@@ -508,23 +526,18 @@ pub fn compile_module_with_opts(
     config.gleam_version = gleam_version.map(|v| GleamVersion::from_pubgrub(v));
 
     let warnings = TypeWarningEmitter::new("/src/warning/wrn.gleam".into(), src.into(), emitter);
-    let inference_result = crate::analyse::ModuleAnalyzerConstructor::<()> {
+    crate::analyse::ModuleAnalyzerConstructor::<()> {
         target,
         ids: &ids,
         origin: Origin::Src,
         importable_modules: &modules,
         warnings: &warnings,
         direct_dependencies: &direct_dependencies,
+        dev_dependencies: &HashSet::from_iter(["dev_dependency".into()]),
         target_support: TargetSupport::Enforced,
         package_config: &config,
     }
-    .infer_module(ast, LineNumbers::new(src), "".into());
-
-    match inference_result {
-        Outcome::Ok(ast) => Ok(ast),
-        Outcome::PartialFailure(ast, errors) => Err((errors.into(), ast.names)),
-        Outcome::TotalFailure(error) => Err((error.into(), Default::default())),
-    }
+    .infer_module(ast, LineNumbers::new(src), "".into())
 }
 
 pub fn module_error(src: &str, deps: Vec<DependencyModule<'_>>) -> String {
@@ -536,7 +549,7 @@ pub fn module_error_with_target(
     deps: Vec<DependencyModule<'_>>,
     target: Target,
 ) -> String {
-    let (error, names) = compile_module_with_opts(
+    let outcome = compile_module_with_opts(
         "themodule",
         src,
         None,
@@ -544,8 +557,14 @@ pub fn module_error_with_target(
         target,
         TargetSupport::NotEnforced,
         None,
-    )
-    .expect_err("should infer an error");
+    );
+
+    let (error, names) = match outcome {
+        Outcome::Ok(_) => panic!("should infer an error"),
+        Outcome::PartialFailure(ast, errors) => (errors.into(), ast.names),
+        Outcome::TotalFailure(errors) => (errors.into(), Default::default()),
+    };
+
     let error = Error::Type {
         names: Box::new(names),
         src: src.into(),
@@ -564,7 +583,7 @@ pub fn internal_module_error_with_target(
     deps: Vec<DependencyModule<'_>>,
     target: Target,
 ) -> String {
-    let (error, names) = compile_module_with_opts(
+    let outcome = compile_module_with_opts(
         "thepackage/internal/themodule",
         src,
         None,
@@ -572,8 +591,14 @@ pub fn internal_module_error_with_target(
         target,
         TargetSupport::NotEnforced,
         None,
-    )
-    .expect_err("should infer an error");
+    );
+
+    let (error, names) = match outcome {
+        Outcome::Ok(_) => panic!("should infer an error"),
+        Outcome::PartialFailure(ast, errors) => (errors.into(), ast.names),
+        Outcome::TotalFailure(errors) => (errors.into(), Default::default()),
+    };
+
     let error = Error::Type {
         names: Box::new(names),
         src: src.into(),
@@ -759,6 +784,7 @@ fn infer_module_type_retention_test() {
         importable_modules: &modules,
         warnings: &TypeWarningEmitter::null(),
         direct_dependencies: &direct_dependencies,
+        dev_dependencies: &HashSet::new(),
         target_support: TargetSupport::Enforced,
         package_config: &config,
     }
@@ -804,6 +830,7 @@ fn infer_module_type_retention_test() {
                                 parameters: vec![TypeValueConstructorField {
                                     type_: generic_var(1),
                                     label: None,
+                                    documentation: None,
                                 }],
                                 documentation: None,
                             },
@@ -812,6 +839,7 @@ fn infer_module_type_retention_test() {
                                 parameters: vec![TypeValueConstructorField {
                                     type_: generic_var(2),
                                     label: None,
+                                    documentation: None,
                                 }],
                                 documentation: None,
                             }
@@ -840,7 +868,8 @@ fn infer_module_type_retention_test() {
             type_aliases: HashMap::new(),
             documentation: Vec::new(),
             contains_echo: false,
-            references: References::default()
+            references: References::default(),
+            inline_functions: HashMap::new(),
         }
     );
 }
@@ -3204,3 +3233,28 @@ pub fn main() {
         vec![("main", "fn() -> Nil")]
     );
 }
+
+// https://github.com/gleam-lang/gleam/issues/4883
+#[test]
+fn variant_inference_ignores_echo() {
+    assert_module_infer!(
+        "
+pub type Wibble {
+  Wibble(wibble: Int, wobble: String)
+  Wobble(a: String, b: Int)
+}
+
+pub fn get_int(w: Wibble) {
+  case echo w {
+    Wibble(..) -> w.wibble
+    Wobble(..) -> w.b
+  }
+}
+",
+        vec![
+            ("Wibble", "fn(Int, String) -> Wibble"),
+            ("Wobble", "fn(String, Int) -> Wibble"),
+            ("get_int", "fn(Wibble) -> Int"),
+        ]
+    );
+}
