@@ -3283,3 +3283,65 @@ pub fn main() {
 "
     );
 }
+
+#[test]
+fn private_opaque_type() {
+    assert_module_error!(
+        "
+opaque type Wibble {
+  Wobble
+}
+"
+    );
+}
+
+#[test]
+fn src_importing_dev_dependency() {
+    assert_module_error!(
+        ("dev_dependency", "some_module", "pub fn main() { Nil }"),
+        "
+import some_module
+
+pub fn main() {
+  some_module.main()
+}
+"
+    );
+}
+
+#[test]
+fn missing_type_constructor_arguments_in_type_annotation_1() {
+    assert_module_error!("pub fn main() -> Result() {}");
+}
+
+#[test]
+fn missing_type_constructor_arguments_in_type_annotation_2() {
+    assert_module_error!(
+        "pub fn main() {
+  let a: Result() = todo
+}"
+    );
+}
+
+#[test]
+fn type_used_as_a_constructor_1() {
+    assert_module_error!("pub fn main() -> Int() {}");
+}
+
+#[test]
+fn type_used_as_a_constructor_2() {
+    assert_module_error!(
+        "pub fn main() {
+  let a: Int() = todo
+}"
+    );
+}
+
+#[test]
+fn type_used_as_a_constructor_with_more_arguments() {
+    assert_module_error!(
+        "pub fn main() {
+  let a: Int(Int, String) = todo
+}"
+    );
+}
