@@ -464,6 +464,28 @@ fn double_unary_integer_literal() {
     assert_warning!("pub fn main() { let _ = --7 }");
 }
 
+#[test]
+fn even_number_of_multiple_integer_negations_raise_a_single_warning() {
+    assert_warning!("pub fn main() { let _ = ----7 }");
+}
+
+#[test]
+fn odd_number_of_multiple_integer_negations_raise_a_single_warning_that_highlights_the_unnecessary_ones()
+ {
+    assert_warning!("pub fn main() { let _ = -----7 }");
+}
+
+#[test]
+fn even_number_of_multiple_bool_negations_raise_a_single_warning() {
+    assert_warning!("pub fn main() { let _ = !!!!True }");
+}
+
+#[test]
+fn odd_number_of_multiple_bool_negations_raise_a_single_warning_that_highlights_the_unnecessary_ones()
+ {
+    assert_warning!("pub fn main() { let _ = !!!!!False }");
+}
+
 // https://github.com/gleam-lang/gleam/issues/2050
 #[test]
 fn double_unary_integer_variable() {
@@ -656,6 +678,86 @@ fn prefer_list_is_empty_over_list_length_lt_1() {
     );
 }
 
+/// https://github.com/gleam-lang/gleam/issues/4861
+#[test]
+fn prefer_list_is_empty_over_list_length_gt_negative_0() {
+    assert_warning!(
+        (
+            "gleam_stdlib",
+            "gleam/list",
+            "pub fn length(_list: List(a)) -> Int { 0 }"
+        ),
+        r#"
+        import gleam/list
+
+        pub fn main() {
+            let a_list = []
+            let _ = list.length(a_list) > 0
+        }
+        "#
+    );
+}
+
+/// https://github.com/gleam-lang/gleam/issues/4861
+#[test]
+fn prefer_list_is_empty_over_negative_0_lt_list_length() {
+    assert_warning!(
+        (
+            "gleam_stdlib",
+            "gleam/list",
+            "pub fn length(_list: List(a)) -> Int { 0 }"
+        ),
+        r#"
+        import gleam/list
+
+        pub fn main() {
+            let a_list = []
+            let _ = 0 < list.length(a_list)
+        }
+        "#
+    );
+}
+
+/// https://github.com/gleam-lang/gleam/issues/4861
+#[test]
+fn prefer_list_is_empty_over_list_length_gt_0() {
+    assert_warning!(
+        (
+            "gleam_stdlib",
+            "gleam/list",
+            "pub fn length(_list: List(a)) -> Int { 0 }"
+        ),
+        r#"
+        import gleam/list
+
+        pub fn main() {
+            let a_list = []
+            let _ = list.length(a_list) > 0
+        }
+        "#
+    );
+}
+
+/// https://github.com/gleam-lang/gleam/issues/4861
+#[test]
+fn prefer_list_is_empty_over_0_lt_list_length() {
+    assert_warning!(
+        (
+            "gleam_stdlib",
+            "gleam/list",
+            "pub fn length(_list: List(a)) -> Int { 0 }"
+        ),
+        r#"
+        import gleam/list
+
+        pub fn main() {
+            let a_list = []
+            let _ = 0 < list.length(a_list)
+        }
+        "#
+    );
+}
+
 /// https://github.com/gleam-lang/gleam/issues/2067
 #[test]
 fn allow_list_length_eq_1() {
@@ -1414,6 +1516,30 @@ pub fn main() {
     );
 }
 
+#[test]
+fn module_used_by_unused_function_is_not_marked_as_unused() {
+    assert_warning!(
+        ("wibble", "pub const a = 1"),
+        "import wibble
+fn wobble() {
+  wibble.a
+}
+"
+    );
+}
+
+#[test]
+fn aliased_module_used_by_unused_function_is_not_marked_as_unused() {
+    assert_warning!(
+        ("wibble", "pub const a = 1"),
+        "import wibble as woo
+fn wobble() {
+  woo.a
+}
+"
+    );
+}
+
 #[test]
 fn calling_function_from_other_module_is_not_marked_unused() {
     assert_no_warnings!(
@@ -4189,3 +4315,219 @@ pub fn main() -> Bool {
 "
     );
 }
+
+#[test]
+fn unused_discard_pattern() {
+    assert_warning!(
+        "pub fn main() {
+  let a = 10
+  let _ = case a {
+    _ as b -> b
+  }
+}
+"
+    );
+}
+
+#[test]
+fn discarded_argument_suggestion_is_not_given_for_different_functions() {
+    assert_no_warnings!(
+        "
+pub fn main(_x) { 1 }
+pub fn wibble() { x }
+"
+    );
+}
+
+#[test]
+fn empty_guard_clause() {
+    assert_warning!(
+        "
+pub fn main() {
+    let wibble = 10
+    case wibble {
+        10 if -> True
+        _ -> False
+    }
+
+}"
+    );
+}
+
+#[test]
+fn impossible_to_reach_integer_segment() {
+    assert_warning!(
+        "
+pub fn main(x) {
+  case x {
+    <<9:size(2)>> -> True
+    _ -> False
+  }
+}"
+    );
+}
+
+#[test]
+fn impossible_to_reach_integer_segment_2() {
+    assert_warning!(
+        "
+pub fn main(x) {
+  case x {
+    <<-9:unsigned>> -> True
+    _ -> False
+  }
+}"
+    );
+}
+
+#[test]
+fn impossible_to_reach_integer_segment_3() {
+    assert_warning!(
+        "
+pub fn main(x) {
+  case x {
+    <<312>> -> True
+    _ -> False
+  }
+}"
+    );
+}
+
+#[test]
+fn impossible_to_reach_integer_segment_4() {
+    assert_warning!(
+        "
+pub fn main(x) {
+  case x {
+    <<-1>> -> True
+    _ -> False
+  }
+}"
+    );
+}
+
+#[test]
+fn multiple_impossible_to_reach_integer_segments() {
+    assert_warning!(
+        "
+pub fn main(x) {
+  case x {
+    <<1, -1, 2, -3>> -> True
+    _ -> False
+  }
+}"
+    );
+}
+
+#[test]
+fn assert_on_impossible_to_reach_integer_segment() {
+    assert_warning!(
+        "
+pub fn main(x) {
+  let assert <<1, -1, 2, -3>> = x
+}"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/4958
+#[test]
+fn unreachable_string_pattern_with_different_encodings() {
+    assert_warning!(
+        r#"
+pub fn wibble(bits) {
+  case bits {
+    <<"\u{0000}a\u{0000}b":utf8>> -> 1
+    // This is the same as the one above, it shouldn't be reachable
+    <<"ab":utf16>> -> 2
+    _ -> 3
+  }
+}"#
+    );
+}
+
+#[test]
+fn unused_recursive_function_argument() {
+    assert_warning!(
+        "
+pub fn main(x: Int) {
+  main(x)
+}
+"
+    );
+}
+
+#[test]
+fn unused_recursive_function_argument_2() {
+    assert_warning!(
+        "
+pub fn main(x, times) {
+  case times {
+    0 -> main(x, 0)
+    _ -> main(x, times - 1)
+  }
+}
+"
+    );
+}
+
+#[test]
+fn unused_recursive_function_with_shadowing() {
+    assert_warning!(
+        "
+pub fn main(x: Int) {
+  let _useless = fn(x) { main(x) }
+  main(x)
+}
+"
+    );
+}
+
+#[test]
+fn unused_recursive_function_inside_anonymous_function() {
+    assert_warning!(
+        "
+pub fn main(x: Int) {
+  let _useless = fn(_) { main(x) }
+  Nil
+}
+"
+    );
+}
+
+#[test]
+fn unused_recursive_function_with_variable_shadowing() {
+    assert_no_warnings!(
+        "
+pub fn main(x: Int) {
+  let x = x * 2
+  main(x)
+}
+"
+    );
+}
+
+#[test]
+fn unused_recursive_function_passing_argument_in_a_different_position_counts_as_using_it() {
+    assert_no_warnings!(
+        "
+pub fn main(x: Int, y) {
+  case y {
+    0 -> main(1, x)
+    _ -> main(0, 0)
+  }
+}
+"
+    );
+}
+
+#[test]
+fn unused_recursive_function_with_shadowed_function() {
+    assert_no_warnings!(
+        "
+pub fn main(x: Int) {
+  let main = fn(x) { x }
+  main(x)
+}
+"
+    );
+}
