@@ -3,11 +3,12 @@ use crate::{
     build::Target,
     diagnostic::{self, Diagnostic, ExtraLabel, Location},
     error::wrap,
+    exhaustiveness::ImpossibleBitArraySegmentPattern,
     type_::{
         self,
         error::{
-            FeatureKind, LiteralCollectionKind, PanicPosition, TodoOrPanic,
-            UnreachablePatternReason,
+            AssertImpossiblePattern, FeatureKind, LiteralCollectionKind, PanicPosition,
+            TodoOrPanic, UnreachablePatternReason,
         },
         expression::ComparisonOutcome,
         pretty::Printer,
@@ -16,6 +17,7 @@ use crate::{
 use camino::Utf8PathBuf;
 use debug_ignore::DebugIgnore;
 use ecow::EcoString;
+use itertools::Itertools;
 use std::{
     io::Write,
     sync::{Arc, atomic::Ordering},
@@ -242,6 +244,17 @@ pub enum DeprecatedSyntaxWarning {
         location: SrcSpan,
     },
 
+    /// If a guard has an empty clause :
+    /// ```gleam
+    /// case wibble {
+    ///     big if -> True
+    ///         ^^ This can be removed.
+    /// }
+    /// ```
+    DeprecatedEmptyClauseGuard {
+        location: SrcSpan,
+    },
+
     DeprecatedTargetShorthand {
         target: Target,
         location: SrcSpan,
@@ -355,7 +368,28 @@ To match on all possible lists, use the `_` catch-all pattern instead.",
                     extra_labels: vec![],
                 }),
             },
-
+            Warning::DeprecatedSyntax {
+                path,
+                src,
+                warning: DeprecatedSyntaxWarning::DeprecatedEmptyClauseGuard { location },
+            } => Diagnostic {
+                title: "Deprecated empty guard syntax".into(),
+                text: wrap(
+                    "This syntax for an empty guard is deprecated. \
+To have a clause without a guard, remove this.",
+                ),
+                hint: None,
+                level: diagnostic::Level::Warning,
+                location: Some(Location {
+                    label: diagnostic::Label {
+                        text: Some("This can be removed.".into()),
+                        span: *location,
+                    },
+                    path: path.clone(),
+                    src: src.clone(),
+                    extra_labels: vec![],
+                }),
+            },
             Warning::DeprecatedSyntax {
                 path,
                 src,
@@ -662,7 +696,11 @@ Hint: You can safely remove it.
                 },
 
                 type_::Warning::UnusedVariable { location, origin } => Diagnostic {
-                    title: "Unused variable".into(),
+                    title: if origin.is_function_parameter() {
+                        "Unused function argument".into()
+                    } else {
+                        "Unused variable".into()
+                    },
                     text: "".into(),
                     hint: origin.how_to_ignore(),
                     level: diagnostic::Level::Warning,
@@ -670,22 +708,46 @@ Hint: You can safely remove it.
                         src: src.clone(),
                         path: path.to_path_buf(),
                         label: diagnostic::Label {
-                            text: Some("This variable is never used".into()),
+                            text: if origin.is_function_parameter() {
+                                Some("This argument is never used".into())
+                            } else {
+                                Some("This variable is never used".into())
+                            },
                             span: *location,
                         },
                         extra_labels: Vec::new(),
                     }),
                 },
-                type_::Warning::UnnecessaryDoubleIntNegation { location } => Diagnostic {
-                    title: "Unnecessary double negation (--) on integer".into(),
-                    text: "".into(),
-                    hint: Some("You can safely remove this.".into()),
+
+                type_::Warning::UnusedRecursiveArgument { location } => Diagnostic {
+                    title: "Unused function argument".into(),
+                    text: wrap(
+                        "This argument is passed to the function when recursing, \
+but it's never used for anything.",
+                    ),
+                    hint: None,
                     level: diagnostic::Level::Warning,
                     location: Some(Location {
                         src: src.clone(),
                         path: path.to_path_buf(),
                         label: diagnostic::Label {
-                            text: None,
+                            text: Some("This argument is never used".into()),
+                            span: *location,
+                        },
+                        extra_labels: vec![],
+                    }),
+                },
+
+                type_::Warning::UnnecessaryDoubleIntNegation { location } => Diagnostic {
+                    title: "Unnecessary double negation (--) on integer".into(),
+                    text: "".into(),
+                    hint: None,
+                    level: diagnostic::Level::Warning,
+                    location: Some(Location {
+                        src: src.clone(),
+                        path: path.to_path_buf(),
+                        label: diagnostic::Label {
+                            text: Some("You can safely remove this.".into()),
                             span: *location,
                         },
                         extra_labels: Vec::new(),
@@ -694,13 +756,13 @@ Hint: You can safely remove it.
                 type_::Warning::UnnecessaryDoubleBoolNegation { location } => Diagnostic {
                     title: "Unnecessary double negation (!!) on bool".into(),
                     text: "".into(),
-                    hint: Some("You can safely remove this.".into()),
+                    hint: None,
                     level: diagnostic::Level::Warning,
                     location: Some(Location {
                         src: src.clone(),
                         path: path.to_path_buf(),
                         label: diagnostic::Label {
-                            text: None,
+                            text: Some("You can safely remove this.".into()),
                             span: *location,
                         },
                         extra_labels: Vec::new(),
@@ -806,7 +868,7 @@ Run this command to add it to your dependencies:
                 }
 
                 type_::Warning::UnreachableCasePattern { location, reason } => {
-                    let text: String = match reason {
+                    let text = match reason {
                         UnreachablePatternReason::DuplicatePattern => wrap(
                             "This pattern cannot be reached as a previous \
 pattern matches the same values.\n",
@@ -815,7 +877,27 @@ pattern matches the same values.\n",
                             "This pattern cannot be reached as it matches on \
 a variant of a type which is never present.\n",
                         ),
+                        UnreachablePatternReason::ImpossibleSegments(_) => wrap(
+                            "This pattern cannot be reached as it contains \
+segments that will never match.\n",
+                        ),
                     };
+
+                    let extra_labels = match reason {
+                        UnreachablePatternReason::DuplicatePattern
+                        | UnreachablePatternReason::ImpossibleVariant => vec![],
+                        UnreachablePatternReason::ImpossibleSegments(segments) => segments
+                            .iter()
+                            .map(|segment| ExtraLabel {
+                                src_info: None,
+                                label: diagnostic::Label {
+                                    text: Some(explain_impossible_segment(segment)),
+                                    span: segment.location(),
+                                },
+                            })
+                            .collect_vec(),
+                    };
+
                     Diagnostic {
                         title: "Unreachable pattern".into(),
                         text,
@@ -828,7 +910,7 @@ a variant of a type which is never present.\n",
                                 text: None,
                                 span: *location,
                             },
-                            extra_labels: Vec::new(),
+                            extra_labels,
                         }),
                     }
                 }
@@ -975,26 +1057,42 @@ hidden from the package's documentation.",
                     }),
                 },
 
-                type_::Warning::AssertAssignmentOnInferredVariant { location } => Diagnostic {
-                    title: "Assertion that will always fail".into(),
-                    text: wrap(
-                        "We can tell from the code above that the value will never match \
+                type_::Warning::AssertAssignmentOnImpossiblePattern { location, reason } => {
+                    let extra_labels = match reason {
+                        AssertImpossiblePattern::InferredVariant => vec![],
+                        AssertImpossiblePattern::ImpossibleSegments { segments } => segments
+                            .iter()
+                            .map(|segment| ExtraLabel {
+                                src_info: None,
+                                label: diagnostic::Label {
+                                    text: Some(explain_impossible_segment(segment)),
+                                    span: segment.location(),
+                                },
+                            })
+                            .collect_vec(),
+                    };
+
+                    Diagnostic {
+                        title: "Assertion that will always fail".into(),
+                        text: wrap(
+                            "We can tell from the code above that the value will never match \
 this pattern and that this code will always crash.
 
 Either change the pattern or use `panic` to unconditionally fail.",
-                    ),
-                    hint: None,
-                    level: diagnostic::Level::Warning,
-                    location: Some(Location {
-                        label: diagnostic::Label {
-                            text: None,
-                            span: *location,
-                        },
-                        path: path.clone(),
-                        src: src.clone(),
-                        extra_labels: vec![],
-                    }),
-                },
+                        ),
+                        hint: None,
+                        level: diagnostic::Level::Warning,
+                        location: Some(Location {
+                            label: diagnostic::Label {
+                                text: None,
+                                span: *location,
+                            },
+                            path: path.clone(),
+                            src: src.clone(),
+                            extra_labels,
+                        }),
+                    }
+                }
 
                 type_::Warning::TodoOrPanicUsedAsFunction {
                     kind,
@@ -1304,6 +1402,25 @@ can already tell whether it will be `True` or `False`.",
                     }),
                 },
 
+                type_::Warning::UnusedDiscardPattern { location, name } => Diagnostic {
+                    title: "Unused discard pattern".into(),
+                    text: format!("`_ as {name}` can be written more concisely as `{name}`"),
+                    level: diagnostic::Level::Warning,
+                    location: Some(Location {
+                        src: src.clone(),
+                        path: path.to_path_buf(),
+                        label: diagnostic::Label {
+                            text: None,
+                            span: SrcSpan {
+                                start: location.start - "_ as ".len() as u32,
+                                end: location.end,
+                            },
+                        },
+                        extra_labels: vec![],
+                    }),
+                    hint: None,
+                },
+
                 type_::Warning::TopLevelDefinitionShadowsImport { location, name } => {
                     let text = format!(
                         "Definition of {name} shadows an imported value.
@@ -1388,6 +1505,26 @@ The imported value could not be used in this module anyway."
     }
 }
 
+fn explain_impossible_segment(segment: &ImpossibleBitArraySegmentPattern) -> String {
+    match segment {
+        ImpossibleBitArraySegmentPattern::UnrepresentableInteger {
+            size,
+            signed,
+            value: _,
+            location: _,
+        } => {
+            let human_readable_size = match size {
+                1 => "1 bit".into(),
+                8 => "1 byte".into(),
+                n if n % 8 == 0 => format!("{} bytes", n / 8),
+                n => format!("{n} bits"),
+            };
+            let sign = if *signed { "signed" } else { "unsigned" };
+            format!("A {human_readable_size} {sign} integer will never match this value")
+        }
+    }
+}
+
 fn pluralise(string: String, quantity: i64) -> String {
     if quantity == 1 {
         string
