@@ -5,8 +5,8 @@ set -eu
 GLEAM_COMMAND=${GLEAM_COMMAND:-"cargo run --quiet --"}
 
 g() {
-	echo "Running: $GLEAM_COMMAND $@"
-	$GLEAM_COMMAND "$@"
+  echo "Running: $GLEAM_COMMAND $@"
+  $GLEAM_COMMAND "$@"
 }
 
 echo Resetting the build directory to get to a known state
@@ -21,19 +21,16 @@ echo Building and running for JavaScript should succeed
 g build --target=javascript
 g run --target=javascript
 
-# TODO: FIXME: Re-enable this one the compiler understand that the previous build
-#       cache cannot be used here.
-#
-# echo Building for Erlang should fail, even if previously a Erlang dependency was built
-# if g build --target=erlang; then
-# 	echo "Expected build to fail"
-# 	exit 1
-# fi
+echo Building for Erlang should fail, even if previously a Erlang dependency was built
+if g build --target=erlang; then
+  echo "Expected build to fail"
+  exit 1
+fi
 
 echo Running for Erlang should fail, even if previously a Erlang dependency was built
 if g run --target=erlang; then
-	echo "Expected run to fail"
-	exit 1
+  echo "Expected run to fail"
+  exit 1
 fi
 
 echo
